#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
2次元プロッタの CUI ファサードクラス
"""
import os
import sys
import thread
from math import pi, sqrt, cos
from numpy import array, arange,meshgrid, zeros, float64
from time import sleep
from vis.WxCom import *
from vis.UtilPlot import *
try:
    import Manyo as mm
    import Manyo.Utsusemi as mu
except:
    pass

"""
brief update history
[inamura 170126] changed to send "map-format" data to M2Plot (for VisualContQ)
[inamura 160627]  added setxlog and setylog
[inamura 160603-2] added to notify "updatedata".
"""

#######################################
#  D2EMatrix
####################################### 
class D2Matrix(object):
    """
    2次元データ用Matrix(粉末タイプ)読み込み
    """
    #########################################
    def ReadMatrix(self, matrix):
        """
        エレメントコンテナマトリックスの読み込み
        @param  matrix  エレメントコンテナマトリックス
        @retval 2次元プロッタの表示データ
        　　　　(datArray,  erArray, X, Y , 1: 斜交データ)
        """
        self.ei = 0.0
        self.runno = ""
        self.datArray = None
        self.erArray = None
        
        try:

            # マトリックスのヘッダーを取得
            mh = matrix.PutHeader()
            ## [inamura 100329]-->
            #self.ei = mh.PutDouble("Ei")
            #self.runno = mh.PutString("RUNNUMBER")
            #type = mh.PutString("SAMPLETYPE")
            ## <-- [inamura 100329]
        except:
            # ManyoLib データ読み出し時に例外発生(不当なデータ)
            raise PlotException('Common', 'C040', ())
            return
        ## [inamura 100329]-->
        # ヘッダーにパラメータがあれば取得
        ## [inamura 100708]-->
        #if (mh.CheckKey("Ei"))*(mh.CheckKey("RUNNUMBER"))*(mh.CheckKey("SAMPLETYPE"))==1:
        if mh.CheckKey("Ei"):
            self.ei = mh.PutDouble("Ei")
        else:
            self.ei = 0.0
        if mh.CheckKey("RUNNUMBER"):
            self.runno = mh.PutString("RUNNUMBER")
        else:
            self.runno = "Unknown"
        if mh.CheckKey("SAMPLETYPE"):
            type = mh.PutString("SAMPLETYPE")
        else:
            type = "Unknown"
        
        #else:
        #    raise PlotException('Common', 'C040', ())
        #    return
        ##<--[inamura 100708]
        ## <-- [inamura 100329]
        ## 試料タイプは粉末か
        #if type != "Powder":
        #    raise PlotException('Common', 'C041', ("Powder",))
        #    return
        try:
            # 2次元マップデータを作る
            ## [inamura 100708]-->
            if type == "Powder":
                X, Y = self._MakeMapData(matrix)
                i_type = 1 ##[inamura 130318]
            else:
                X, Y = self._MakeNormal2DMap(matrix)
                i_type = 0 ##[inamura 130318]
            ##<--[inamura 100708]
        except:
            raise
        else:
            # マップデータを返す
            #return (self.datArray, self.erArray, X, Y, 1) ##[inamura 130318]
            return (self.datArray, self.erArray, X, Y, i_type)

    #########################################
    def GetHeaderInfo(self):
        """
        エレメントコンテナマトリックスのヘッダー情報取得
        @param  無し
        @retval ラン番号と入射エネルギー
        """ 
        return self.runno, self.ei       

    #########################################
    def _MakeMapData(self, matrix):
        """
        エレメントコンテナマトリックスの読み込み
        @param  matrix  エレメントコンテナマトリックス
        @retval 2次元プロッタの表示データ
        　　　　(datArray,  erArray, X, Y , 1: 斜交データ)
        """
        ##[inamura 100708]-->
        #if type(matrix)==type(ElementContainerArray()):
        #if isinstance( matrix, mm.ElementContainerMatrix ): ##[inamura 130619]
        if isinstance( matrix, mm.ElementContainerArray ): ##[inamura 130619]
            eca = matrix
            matrix = mm.ElementContainerMatrix()
            matrix.Add(eca)
        ##<--[inamura 100708]
        # 全データ共通のはずなので、最初の1個から取得
        e0 = matrix(0,0)
        ebin = e0.PutXList()
            
        # アレイ の数を取得 (1?)
        numAr = matrix.PutSize()
        pols = []
        # アレイ 数だけ繰り返す
        for i in range(numAr):
            # マトリックスから 1アレイ分のデータ(エレメントアレイ)を読み出す
            ea = matrix.PutPointer(i)
            # エレメントコンテナ数を取り出す
            nEc = ea.PutSize()
            # エレメントアレイから、各 エレメントコンテナを取り出す
            for j in range(nEc):
                # エレメントコンテナのヘッダ取得
                ec = ea(j)
                ech = ec.PutHeader()
                ##[inamura 100122]->
                # MASKED=1かどうか
                if ((ech.CheckKey("MASKED")==1) and (ech.PutInt4("MASKED")==1)):
                    continue
                ##<-[inamura 100122]
                # 座標角(度)を取得
                pol = ech.PutDoubleVector("PixelPolarAngle")
                polr = pol[0]*pi/180.0

                # リストに追加
                pols.append(polr)

        if len(pols) < 2:
            raise PlotException('Common', 'C040', ())
            return
        
        pnum = len(pols)
        polr = pols[pnum-1]-pols[pnum-2]+pols[pnum-1]
        pols.append(polr)
        hfGap = (pols[1]-pols[0])/2.0
        # 最初のPSD
        theta = pols[0] - hfGap
        # 1本のPSD の角度より1.5倍以上あるとき、バンクギャップとみなす
        gapTh = hfGap*3.0
        
        thetas = [theta]
        gaps = []

        # PSD 両端部の散乱角を計算
        for i in range(pnum):
            # 確度の差分を求める
            gap = pols[i+1]-pols[i]
            # バンクギャップでなければ
            if gap < gapTh:
                hfGap = gap/2.0
                theta = pols[i] + hfGap
                thetas.append(theta)
                gaps.append(False)
            # バンクギャップ
            else:
                # 前の半値から
                theta = pols[i] + hfGap
                thetas.append(theta)
                gaps.append(False)
                theta = pols[i+1] - hfGap
                thetas.append(theta)
                gaps.append(True)
            ##[inamura 091219]->
            #角度差は常に一定ではないのでその都度ギャップ用閾値を作成する
            gapTh = hfGap*3.0
            ##<-[inamura 091219]
                
        enL = []
        datL = []
        erL = []
        # バンクギャップ用のマスク値作成
        mskYY = []
        mskEr = []
        for i in range(len(ebin)-1):
            mskYY.append(MASKVALUE)
            mskEr.append(0.0)
        
        k = 0
        # アレイ 数だけ繰り返す
        for i in range(numAr):
            # マトリックスから 1アレイ分のデータ(エレメントアレイ)を読み出す
            ea = matrix.PutPointer(i)
            ##[inamura 091219]->
            #ギャップの数に応じてデータを作成する
            for j in range(len(gaps)):
                #バンクギャップか
                if gaps[j]:
                    datL.append(mskYY)
                    erL.append(mskEr)
                else:
                    ec = ea.PutPointer(k)
                    yy = ec.PutYList()
                    er = ec.PutEList()

                    datL.append(yy)
                    erL.append(er)
                    k +=1
            """
            # エレメントコンテナ数を取り出す
            nEc = ea.PutSize()
            # エレメントアレイから、各 エレメントコンテナを取り出す
            for j in range(nEc):
                # バンクギャップか
                if gaps[k]:
                    # マスク値を入れる
                    datL.append(mskYY)
                    erL.append(mskEr)
                    k += 1
                
                # エレメントコンテナ読み出し
                ##[inamura 091210]-->
                #ec = ea.PutPointer(j)
                ec = HistogramBinToPoint(ea.PutPointer(j))
                ##<--[inamura 091210]
                yy = ec.PutYList()
                er = ec.PutEList()
                
                # リストに追加
                datL.append(yy)
                erL.append(er)
                k += 1
            """
            ##<-[inamura 091219]
        # 全点 0 のアレイを作成
        self.datArray =zeros((len(mskYY),len(datL)), float64)
        self.erArray =zeros((len(mskYY),len(datL)), float64)

        # 縦横を逆にして取り込む
        for i in range(len(mskYY)):
            for j in range(len(datL)):
                self.datArray[i][j] = datL[j][i]
                self.erArray[i][j] = erL[j][i]
                if erL[j][i]<0.0:                  #[inamura 170131]
                    self.datArray[i][j]=MASKVALUE

        # マップデータ用の X Y グリッドを作る
        return self._MakeXYGrid(ebin, thetas)

    #########################################    
    def _MakeXYGrid(self, ebin, thetas):
        """
        XグリッドとYグリッドを計算
        @param  ebin　エネルギーbin
        @param thetas 散乱角
        @retval XグリッドとYグリッド
        """ 

        x = arange(0.0,len(thetas) , 1.0)
        y = array(ebin)
        
        X,Y = meshgrid(x, y)

        ki = sqrt(self.ei/2.072)
        ki2 = ki*ki

        for i in range(len(ebin)):

            hw = ebin[i]
            j = 0
            for theta in thetas:
                # 角度とエネルギーから散乱後の波数ベクトル計算
                kf = sqrt((self.ei-hw)/2.072)
                # Q を計算
                q =sqrt( ki2 + kf*kf - 2.0*ki*kf*cos(theta))
                X[i][j]= q
                j += 1
            
        return (X,Y)    

##[inamura 100708]-->
    #########################################
    def _MakeNormal2DMap(self,eca_in):
        """
        指定されたPSD のデータを返す
        @param  psd  psd No.
        @retval PSD のデータ
        """
        #if type(eca)!=type(ElementContainerArray()):
        ##[inamura 130619]-->
        #if not isinstance( eca, mm.ElementContainerArray ):
        #    raise
        if isinstance( eca_in, mm.ElementContainerArray ):
            eca = eca_in
        elif isinstance( eca_in, mm.ElementContainerMatrix ):
            eca = eca_in(0)
        else:
            msg = "M2Plot::_MakeNormal2DMap > argument is not ECA nor ECM"
            print( msg )
            raise UserWarning,msg
        ##<--[inamura 130619]
        numPixel = eca.PutTableSize()
        # 中心のエレメントコンテナのTOFを取得
        ec0 = eca(0) 
        tof = ec0.PutX()
        # リビニング後のY データの個数
        znum = len(tof)-1
       
        # リビニング後のカウント数アレイ用に全点 0 のアレイを作成
        self.datArray = zeros((znum, numPixel), float64)
        self.erArray  = zeros((znum, numPixel), float64)
        
        ## [inamura 150330]-->
        #hh_p = eca_in.PutHeaderPointer()
        #isHist = True
        #if (hh_p.CheckKey("isHistogram")==1) and (hh_p.PutInt4("isHistogram")==0): # isHistogram is existed and is 0 
        #    isHist = False
        #if (isHist):
        #    pass
        #else:
        #    hh_p = eca.PutHeaderPointer()
        #    if (hh_p.CheckKey("isHistogram")==1) and (hh_p.PutInt4("isHistogram")==1): # isHistogram is existed and is 1 
        #        isHist = True
        ## <--[inamura 150330]
        
        # 1検出器分の強度データを取り出す
        ## [inamura 100712]-->
        x_list = []
        for i in range(numPixel):
            # エレメントコンテナ取得
            ec = eca(i)
            hh = ec.PutHeader()
            if hh.CheckKey("XRANGE"):
                x_list.append( hh.PutDoubleVector("XRANGE")[0] )
            else:
                x_list.append(i)
            ##[inamura 150330]-->
            #if (isHist) or ( (hh.CheckKey("isHistogram")==1) and (hh.PutInt4("isHistgram")==1) ):
            #    print "@@@@ isHist is True" #[inamura 150515]
            #    ec = mu.HistogramBinToPoint( ec )
                
            yy = ec.PutYList()   
            ee = ec.PutEList()
            
            ##<--[inamura 150330]    
            # カウント数アレイとエラー値アレイへ格納 
            for j in range(len(yy)):
                self.datArray[j,i]=yy[j]
                self.erArray[j,i]=ee[j]
                if ee[j]<0.0:                  #[inamura 170131]
                    self.datArray[j,i]=MASKVALUE

        if hh.CheckKey("XRANGE"):
            x_list.append( hh.PutDoubleVector("XRANGE")[1] )
        else:
            x_list.append( numPixel )

        x = array( x_list )
        #x = arange(0, numPixel+1,1)
        ##<--[inamura 100712]
        return meshgrid(x, tof)
    
##<--[inamura 100708]

    
#######################################
#  M2Plot
####################################### 
class M2Plot(object) :
    """
    2次元プロッタのファサードクラス
    Python インタープリタ からプロッタを起動し
    制御するためのユーザインターフェイスを提供
    """
    # 2Dプロッタ通信クラスのインスタンスリスト
    plotList = []
    # plotList のロック用
    lck = thread.allocate_lock()
  
    #########################################
    def __init__(self, matrix=None, closeFlag = True):
        """
        コンストラクタ
        @param  matrix  2Dプロッタに表示するデータ、省略可
        　　　　　　　(エレメントコンテナマトリックス)
        @closeFlag   親プロセス終了時に、プロッタをクローズするか
        　　　　　　 否かのフラグ
        @retval 無し
        """
        self.plot = None
        # 入力チェッククラスのインスタンスを取得、メッセージ出力先は標準出力を指定
        self.val = Validator(None)
        # テンポラリファイルクラスのインスタンス取得
        self.tp = TempFile()
        # プロッタの作成
        self.NewPlot(matrix, closeFlag)
        
    ######################################### 
    def NewPlot(self, matrix, closeFlag = True): 
        """
        プロッタを追加し、カレントプロッタとする
        @param  data  プロッタに表示するデータ、省略可
        　　　　       ???
        @param closeFlag  親プロセス終了時に、子プロセスを終了させるかどうかのフラグ
        @retval 無し
        """    
        # プロッタNo.を取得
        pno = len(self.plotList) +1
        strPno = "%d" % pno
        self.pno = pno #[inamura 170126]
        
        if closeFlag == True:
            cflag = "True"
        else:
            cflag = "False"
        
        arg = ["D2Vis.py",strPno, cflag]

        # プロセスID とプロッタ番号からポート番号を生成
        pxx = (os.getpid() % 500)*20
        portno = 20000+ pxx + pno

        # GUI通信クラスのインスタンス取得
        p = CommWx()
        
        # プロセス生成と通信接続に成功したなら
        if p.NewComm(portno, arg):
            # 旧カレントプロッタのアイコン化要求
            self._IconizeCurrent()
            
            self.lck.acquire()
            # 追加したプロッタをリストへ追加
            self.plotList.append(p)
            # ロックの解放
            self.lck.release()
                
            # 追加したプロッタをカレントプロッタとする。   
            self.plot = p
            # データを設定       
            self.ChangeMap(matrix, True)
                
        else:
            print "\nError!!  Cannot create a new plotter!!."
    
    ######################################### 
    def ChangePlot(self, plot=None): 
        """
        カレントプロッタを変更する
        @param  plot  カレントにするプロッタ番号
        @retval 無し
        """
        # 引数が指定されていない場合
        if not self.val.ValidateObject(plot, "ChangePlot"):
            # エラー終了
            return   
        # 引数チェック
        if self.val.ValidateInteger(plot, "Argument", 1, len(self.plotList)):
            # 指定されたプロッタが閉じられていなければ
            if self.plotList[plot-1].IsAliveComm():
                # 旧カレントプロッタのアイコン化要求    
                self._IconizeCurrent()
                # カレントプロッタを変更
                self.plot = self.plotList[plot-1]
                # 新カレントプロッタの復元要求    
                self.plot.PutCommand("doiconize 0\n")
            # 既に閉じている場合は
            else:
                # エラーメッセージを表示
                print "\nThe 2D plotter(%d) has been already closed." % plot
     
    ######################################### 
    def _IconizeCurrent(self): 
        """
        カレントプロッタをアイコン化する
        @param  無し
        @retval 無し
        """             
        # 先にカレントプロッタがあれば
        if self.plot != None:
            # カレントプロッタが閉じられていなければ
            if self.plot.IsAliveComm():
                # 旧カレントプロッタのアイコン化要求
                self.plot.PutCommand("doiconize 1\n")

    #########################################         
    def ChangeMap(self, data=None, new = False):
        """
        プロッタのデータを変更
        @param  data  変更するデータ
        　　　　エレメントコンテナマトリックス
        @param  new  新規プロッタ
        @retval 無し
        """ 
        # 引数が指定されていない場合
        #if not self.val.ValidateObject(matrix, "ChangeData"):
            # エラー終了
        #    return   

        ##[inamrua 170126]-->
        isContainer = True
        if data==None:
            head = mm.HeaderBase()
            head.Add("RUNNUMBER","Dummy data")
            data = mm.ElementContainerMatrix(head)
            eca = mm.ElementContainerArray(head)
            a_ec = mm.ElementContainer()
            #a_ec.Add( "X", range(21) )
            a_ec.Add( "X", range(1,21) )
            a_ec.Add( "Y", range(20) )
            a_ec.Add( "E", range(20) )
            a_ec.SetKeys("X","Y","E")
            for i in range(10):
                rng = mm.MakeDoubleVector(2)
                rng[0] = 0.5 + float(i)*1.0
                rng[1] = 0.5 + float(i+1)*1.0
                hh = a_ec.PutHeaderPointer()
                if hh.CheckKey("XRANGE")==1:
                    hh.OverWrite("XRANGE",rng)
                else:
                    a_ec.AddToHeader("XRANGE",rng)
                eca.Add( mm.ElementContainer( a_ec ) )
            data.Add( eca )
            ecType = 0
        elif isinstance( data, mm.ElementContainerArray ):
            #print "Change Map: ElementContainerArray"
            # Matrix に変更
            ecType = 1
            # ヘッダーがあるか
            try:
                data.PutHeader()
            except:
                # ヘッダーを追加
                head=mm.HeaderBase()
                head.Add("RUNNUMBER","Unknown")
                data.SetHeader(head)

        elif isinstance(data,mm.ElementContainerMatrix):
            #print "Change Map: ElementContainerMatrix"
            ecType = 0
        elif type(data)==tuple:
            isContainer = False

        else:
            return
        
        if (isContainer):
            temppath = self.tp.GetTempName(".srlz")
            if temppath == None:
                # 異常終了
                print "\nError!! Cannot make a temporary file."   
                return
            # シリアライズ
            crw = mm.WriteSerializationFileBinary(temppath)
            crw.Save(data)
            del crw
            # コマンド作成(改行コード付)
            strCmd = "changedata %s %d\n" % (temppath, ecType)
            # コマンドの送信要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)
            
        else:
            temppath = self.tp.GetTempName(".map")
            if temppath == None:
                # 異常終了
                print "\nError!! Cannot make a temporary file."   
                return
            
            ret = self.tp.MakeTempFile(data, temppath)
            # 正常終了か
            if ret == 0:
                # コマンド作成(改行コード付)
                strCmd = "changedata %s\n" % temppath
                # コマンドの送信要求をカレントプロッタへ送る
                self.plot.PutCommand(strCmd)
            else:
                # 異常終了
                print "\nError!! Cannot make a temporary file."
        """
        # テンポラリファイル名称を取得
        temppath = self.tp.GetTempName()
       
        if temppath == None:
            # 異常終了
            print "\nError!! Cannot make a temporary file."   
            return

        crw = mm.WriteSerializationFileBinary(temppath)
        
        ##[inamura 140821]-->
        if data==None:
            head = mm.HeaderBase()
            head.Add("RUNNUMBER","Dummy data")
            data = mm.ElementContainerMatrix(head)
            eca = mm.ElementContainerArray(head)
            a_ec = mm.ElementContainer()
            #a_ec.Add( "X", range(21) )
            a_ec.Add( "X", range(1,21) )
            a_ec.Add( "Y", range(20) )
            a_ec.Add( "E", range(20) )
            a_ec.SetKeys("X","Y","E")
            for i in range(10):
                rng = mm.MakeDoubleVector(2)
                rng[0] = 0.5 + float(i)*1.0
                rng[1] = 0.5 + float(i+1)*1.0
                hh = a_ec.PutHeaderPointer()
                if hh.CheckKey("XRANGE")==1:
                    hh.OverWrite("XRANGE",rng)
                else:
                    a_ec.AddToHeader("XRANGE",rng)
                eca.Add( mm.ElementContainer( a_ec ) )
            data.Add( eca )
        ##<--[inamura 140821]
        
        # Array データであれば
        if type(data)==type(mm.ElementContainerArray()):
            print "Change Map: ElementContainerArray"
            # Matrix に変更
            ecType = 1
            # ヘッダーがあるか
            try:
                data.PutHeader()
            except:
                # ヘッダーを追加
                head=mm.HeaderBase()
                head.Add("RUNNUMBER","Unknown")
                data.SetHeader(head)

        elif type(data)==type(mm.ElementContainerMatrix()):
            print "Change Map: ElementContainerMatrix"
            ecType = 0
            
        else:
            print "Error data!!"
            print "Accept only ElementContainerMatrix or ElementContainerArray only."
            return  
            
        # シリアライズ
        crw.Save(data)
        del crw    #[inamura 130617]
        # コマンド作成(改行コード付)
        strCmd = "changedata %s %d\n" % (temppath, ecType)
        # コマンドの送信要求をカレントプロッタへ送る
        self.plot.PutCommand(strCmd)
        
        # Manyo データからPython データタイプへ変換
        #d2 = D2Matrix()
        #try:
        #    mapXY = d2.ReadMatrix(matrix)
        #except PlotException, ex:
        #    PlotMessage(None, ex)
        #    return
        ## メインタイトルとサブタイトルを作る
        #
        #runno, ei = d2.GetHeaderInfo()
        #main = "Run no.: %s" % runno
        ###[inamura 100708]-->
        ##sub = "Ei (meV): %.2f" % ei
        #if ei != 0.0:
        #    sub = "Ei (meV): %.2f" % ei
        #else:
        #    if matrix.PutHeader().CheckKey("COMMENTS"):
        #        sub = matrix.PutHeader().PutString("COMMENTS")
        #    else:
        #        sub = ""
        #
        #if new:
        #    hh = matrix.PutHeader()
        #    xlabel = "Q(1/Angstrom)"
        #    ylabel = "Energy(meV)"
        #    zlabel = "Neutrons"
        #    if hh.CheckKey("XLABEL"):
        #        xlabel = hh.PutString("XLABEL")
        #    if hh.CheckKey("YLABEL"):
        #        ylabel = hh.PutString("YLABEL")
        #    if hh.CheckKey("ZLABEL"):
        #        zlabel = hh.PutString("ZLABEL")
        #    
        #    #object = [mapXY, main, sub, "Q(1/Angstrom)", "Energy(meV)","Neutrons"]
        #    object = [mapXY, main, sub, xlabel, ylabel, zlabel]
        ###<--[inamura 100708]
        #else:
        #    object = [mapXY, main, sub]
        #
        ## オブジェクトをシリアライズしてテンポラリファイルへ書き込む
        #ret = self.tp.MakeTempFile(object, temppath)
        ## 正常終了か
        #if ret == 0:
        #    # コマンド作成(改行コード付)
        #    strCmd = "changedata %s\n" % temppath
        #    # コマンドの送信要求をカレントプロッタへ送る
        #    self.plot.PutCommand(strCmd)
        #else:
        #    # 異常終了
        #    print "\nError!! Cannot make a temporary file." 
        """
        ##<--[inamrua 170126]

    #[inamrua 160603-2]-->
    #########################################         
    def UpdateMap(self, data=None, new = False):
        """
        プロッタのデータを変更
        @param  data  変更するデータ
        　　　　エレメントコンテナマトリックス
        @param  new  新規プロッタ
        @retval 無し
        """ 
        # テンポラリファイル名称を取得
        temppath = self.tp.GetTempName()
       
        if temppath == None:
            # 異常終了
            print "\nError!! Cannot make a temporary file."   
            return

        crw = mm.WriteSerializationFileBinary(temppath)
        
        ##[inamura 140821]-->
        if data==None:
            head = mm.HeaderBase()
            head.Add("RUNNUMBER","Dummy data")
            data = mm.ElementContainerMatrix(head)
            eca = mm.ElementContainerArray(head)
            a_ec = mm.ElementContainer()
            a_ec.Add( "X", range(21) )
            a_ec.Add( "Y", range(20) )
            a_ec.Add( "E", range(20) )
            a_ec.SetKeys("X","Y","E")
            for i in range(10):
                eca.Add( mm.ElementContainer( a_ec ) )
            data.Add( eca )
        ##<--[inamura 140821]
        
        # Array データであれば
        if type(data)==type(mm.ElementContainerArray()):
            print "Update Map: ElementContainerArray"
            # Matrix に変更
            ecType = 1
            # ヘッダーがあるか
            try:
                data.PutHeader()
            except:
                # ヘッダーを追加
                head=mm.HeaderBase()
                head.Add("RUNNUMBER","Unknown")
                data.SetHeader(head)

        elif type(data)==type(mm.ElementContainerMatrix()):
            print "Update Map: ElementContainerMatrix"
            ecType = 0
            
        else:
            print "Error data!!"
            print "Accept only ElementContainerMatrix or ElementContainerArray only."
            return  
            
        # シリアライズ
        crw.Save(data)
        del crw    #[inamura 130617]
        # コマンド作成(改行コード付)
        strCmd = "updatedata %s %d\n" % (temppath, ecType)
        # コマンドの送信要求をカレントプロッタへ送る
        self.plot.PutCommand(strCmd)
        
    #<--[inamrua 160603-2]

 
        
    #########################################         
    def SetLog(self, flag=True):
        """
        ログモード設定
        @param  flag  ログモード True or False
        @retval 無し
        """ 
        # 引数チェック
        if self.val.ValidateBoolean(flag, "Argument"):
            # コマンド作成
            strCmd = "setlog %d\n" % flag
            # コマンドの送信要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)

    #[inamura 160627]-->
    #########################################         
    def SetXLog(self, flag=True):
        """
        ログモード設定
        @param  flag  ログモード True or False
        @retval 無し
        """ 
        # 引数チェック
        if self.val.ValidateBoolean(flag, "Argument"):
            # コマンド作成
            strCmd = "setxlog %d\n" % flag
            # コマンドの送信要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)

    #########################################         
    def SetYLog(self, flag=True):
        """
        ログモード設定
        @param  flag  ログモード True or False
        @retval 無し
        """ 
        # 引数チェック
        if self.val.ValidateBoolean(flag, "Argument"):
            # コマンド作成
            strCmd = "setylog %d\n" % flag
            # コマンドの送信要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)

    #<--[inamura 16027]
    #########################################         
    def SetWithPlot(self, flag=None, n=3):
        """
        プロッタ送信モード設定
        @param  flag  送信モード True or False
        @param   n  1: Xのみ  2: Yのみ 3: 両方
        @retval 無し
        """ 
        # 引数1チェック
        if not self.val.ValidateBoolean(flag, "Argument1"):
            return
        # 引数2 チェック
        if self.val.ValidateInteger(n, "Argument2", 1,3):
            # コマンド作成
            strCmd = "setwith %d %d\n" % (flag,n)
            # コマンドの送信要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)
        
    #########################################         
    def SetZScale(self, z0=None, z1=None ):
        """
        自動スケールモード設定
        @param  z0 X軸始値
        @param  z1 X軸終値
        @retval 無し
        """ 
        # 引数無しか
        if z0==None and z1==None:
            # 自動モード(引数無し)コマンドの送信要求
            self.plot.PutCommand("setzscale\n")
            return
        
        if z1 == None:
            print "\nSetZScale takes none or 2 arguments." 
            return
        
        # 引数1チェック
        if not self.val.ValidateNumeric(z0, "Argument1",0.0):
            # エラー終了
            return 
        # 引数2チェック
        if not self.val.ValidateNumeric(z1, "Argument2" ,0.0):
            # エラー終了
            return 
        # 範囲チェック
        if z1 > z0:   
            # コマンド文字列作成
            strCmd = "setzscale %.3f %.3f\n" % (z0, z1)
            # コマンドの送信要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)
        else:
            print "\nThe argument2 must be larger then the argument1."
                                        
    #########################################         
    def Close(self, plot=None):
        """
        プロッタ画面を閉じる
        @param  pno  プロッタ番号
        @retval 無し
        """ 
        # カレントプロッタのクローズ要求か
        if plot == None:
            self.plot.PutCommand("close\n" )
            
        # 入力チェックがOKか
        if self.val.ValidateInteger (plot, "Argument(Plotter Number)", 0, len(self.plotList)):
            # 全画面クローズ要求か
            if plot == 0:
                # 全プロッタに対し
                for pl in self.plotList:
                    # 生存していれば
                    if pl.IsAliveComm():
                        # クローズ要求を送信
                        pl.PutCommand("close\n" )
            else:
                pnum=int(plot)
                # 指定プロッタのインスタンス取得
                pl = self.plotList[pnum-1]
                # クローズ要求を送信
                pl.PutCommand("close\n" )


    #########################################         
    def SliceX(self, y=None, diff = 0.0, bin = 0.0):
        """
        Y値を指定してX方向にスライスする
        @param  y    Y値
        @param  diff 積算範囲　(Y±diff の範囲で積算)
        @param  bin  リビニング値　(0.0のときリビニングしない)
        @retval 無し
        """ 
        # 引数1チェック
        if y == None:
            print "\nSliceX needs at least one argument."
            return
        if not self.val.ValidateNumeric(y, "Argument1"):
            # エラー終了
            return 
        # 引数2チェック
        if not self.val.ValidateNumeric(diff, "Argument2" , 0.0):
            # エラー終了
            return 
        # 引数3チェック
        if self.val.ValidateNumeric(bin, "Argument3" , 0.0):
            strCmd = "slicex %.3f %.3f %.3f\n" % (y, diff, bin)
            ret = self.plot.PutCommand(strCmd)
    
    #########################################         
    def SliceY(self, x=None, diff = 0.0, bin = 0.0):
        """
        X値を指定してY方向にスライスする
        @param  x    X値
        @param  diff 積算範囲　(X±diff の範囲で積算)
        @param  bin  リビニング値　(0.0のときリビニングしない)
        @retval 無し        """ 
        # 引数1チェック
        if x == None:
            print "\nSliceY needs at least one argument."
            return
        if not self.val.ValidateNumeric(x, "Argument1"):
            # エラー終了
            return 
        # 引数2チェック
        if not self.val.ValidateNumeric(diff, "Argument2" , 0.0):
            # エラー終了
            return 
        # 引数3チェック
        if self.val.ValidateNumeric(bin, "Argument3" , 0.0, 1000.0):
            strCmd = "slicey %.3f %.3f %.3f\n" % (x, diff, bin)
            ret = self.plot.PutCommand(strCmd)
            
    #########################################         
    def SliceXRange(self, y0=None, y1=None, fract=1, bin = 0.0):
        """
        Xデータを切り取るYの範囲を指定        @param y0  X値の始値
        @param  y1  Y値の終値
        @param  fract  分割数
        @param  bin 　ビン値
        @retval 無し        """
        # 引数の有無をチェック
        if y0 == None or y1==None:
            print "\nSliceXRange needs at least two arguments."
            return
        
        # 引数1チェック
        if not self.val.ValidateNumeric(y0, "Argument 1"):
            # エラー終了
            return 
        # 引数2チェック
        if not self.val.ValidateNumeric(y1, "Argument 2" ):
            # エラー終了
            return 

        # 引数3チェック
        if not self.val.ValidateInteger(fract, "Argument 3" , 1,50):
            # エラー終了
            return 
        # 引数4チェック
        if self.val.ValidateNumeric(bin, "Argument 4" , 0.0, 1000.0):
            # 終値 >= 始値チェック
            if y1 > y0:
                strCmd = "slicexrange %.3f %.3f %d %.3f\n" % (y0, y1, fract, bin)
                ret = self.plot.PutCommand(strCmd)
            else:
                print "\nThe argument2 must be larger then the argument1." 
    
    #########################################         
    def SliceYRange(self, x0=None, x1=None, fract=1, bin = 0.0):
        """
        Y データを切り取るXの範囲を指定        @param  x0  X値の始値
        @param  x1  X値の終値
        @param  fract  分割数
        @param  bin 　ビン値
        @retval 無し        """
        # 引数の有無をチェック
        if x0 == None or x1==None:
            print "\nSliceYRange needs at least two arguments."
            return
        # 引数1チェック
        if not self.val.ValidateNumeric(x0, "Argument 1"):
            # エラー終了
            return 
        # 引数2チェック
        if not self.val.ValidateNumeric(x1, "Argument 2" ):
            # エラー終了
            return 

        # 引数3チェック
        if not self.val.ValidateInteger(fract, "Argument 3" , 1,50):
            # エラー終了
            return 
        # 引数4チェック
        if self.val.ValidateNumeric(bin, "Argument 4" , 0.0, 1000.0):
            # 終値 >= 始値チェック
            if x1 > x0:
                strCmd = "sliceyrange %.3f %.3f %d %.3f\n" % (x0, x1, fract, bin)
                ret = self.plot.PutCommand(strCmd)
            else:
                print "\nThe argument2 must be larger then the argument1."  

    #########################################         
    def AddToPlot(self, n=3):
        """
        表示中のデータをプロッタへ送信
        @param  n  1: Xのみ  2: Yのみ 3: 両方
        @retval 無し
        """
        # 引数チェック
        if self.val.ValidateInteger(n, "Argument", 1,3):
            strCmd = "add %d\n" % n
            # データ追加要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd) 
        
    #########################################         
    def ClearPlot(self, n=3):
        """
        プロッタをクリア
        @param  n  1: Xのみ  2: Yのみ 3: 両方
        @retval 無
        """
        # 引数チェック
        if self.val.ValidateInteger(n, "Argument", 1,3):
            strCmd = "clear %d\n" % n
            # データ追加要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)
             
    #########################################         
    def SetXLabel(self, xlabel=""):
        """
        X軸のラベル設定
        @param  xlabel　横軸のラベル文字列
        @retval 無し
        """
        # 引数チェック
        if self.val.ValidateString(xlabel, "Argument", 0,50):
            # コマンド作成
            strCmd = "xlabel %s\n" % xlabel
            # 座標ラベル変更要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)

    #########################################         
    def SetYLabel(self, ylabel=""):
        """
        Y軸のラベル設定
        @param  xlabel　横軸のラベル文字列
        @retval 無し
        """
        # 引数チェック
        if self.val.ValidateString(ylabel, "Argument", 0,50):
            # コマンド作成
            strCmd = "ylabel %s\n" % ylabel
            # 座標ラベル変更要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)

    #########################################         
    def SetZLabel(self, zlabel=""):
        """
        Z軸のラベル設定
        @param  zlabel　横軸のラベル文字列
        @retval 無し
        """
        # 引数チェック
        if self.val.ValidateString(zlabel, "Argument", 0,50):
            # コマンド作成
            strCmd = "zlabel %s\n" % zlabel
            # 座標ラベル変更要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)
            
    #########################################         
    def SetMainTitle(self, main=""):
        """
        メインタイトル設定
        @param  main　メインタイトル
        @retval 無し
        """
        # 引数チェック
        if self.val.ValidateString(main, "Argument", 0,50):
            # コマンド作成
            strCmd = "maintitle %s\n" % main
            # タイトル変更要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd) 
    
    #########################################         
    def SetSubTitle(self, sub=""):
        """
        サブタイトル設定
        @param  sub　サブタイトル
        @retval 無し
        """
        # 引数チェック
        if self.val.ValidateString(sub, "Argument", 0,50):
            # コマンド作成
            strCmd = "subtitle %s\n" % sub
            # タイトル変更要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)               
                                     
    #########################################         
    def Print(self):
        """
        カレントプロッタを印刷する
        @param  無し
        @retval 無し
        """ 
        # 印刷要求をカレントプロッタへ送る
        answer = self.plot.PutCommand("print\n" ) 
        
    #########################################         
    def Save(self, fname=None):
        """
        画像ファイルの保存
        @param  ファイル名
        @retval 無し
        """
        # 引数が指定されていない場合
        if not self.val.ValidateObject(fname, "Save"):
            # エラー終了
            return   
        # 引数チェック
        if not self.val.ValidateString(fname, "Argument", 4,256):
            return
        
        # ファイル名取り出し
        filename = os.path.basename(fname)
        path, ext = os.path.splitext(filename)
        # 拡張子を除くファイル名の長さは1以上か
        if len(path) > 1:
            # サポートしている拡張子か
            if ext==".png" or ext==".ps" or ext==".eps" :
                strCmd = "save " + fname + '\n'
                # 画像保存要求をカレントプロッタへ送る
                self.plot.PutCommand(strCmd) 
            else:
                print "\nError!!  Extention must be png, ps, or eps."
        else:
            print "\nError!! Invalid file name."

#####################################################################
if __name__ == '__main__':
    """
    以下はテスト用のスクリプト

    """
    M2Plot(None,False)
    
    #import ECMSerialize as ECM
    #ecms = ECM.ECMSerialize()
    #DAT = ecms.Decode("/home/mlfdev/PowderData120207.dmp")
    #p = M2Plot(DAT, False)
#    import Read2D
#    # レガシィファイルを読み込んで、エレメントコンテナマトリックスを作成
#    dat = Read2D.Read2D("dummy.spe", "dummy.phx", 70.0)
#    mat = dat.GetMatrix()
#   
#    p=M2Plot(mat)
#    sleep(1)
#    p.SetLog(True)
#    p.Print()
#    p.Save("2dplot.ps")
#    p.NewPlot(mat)
#    p.NewPlot(mat)
#    p.Close(2)
#    p.SetMainTitle("Title set")
#    p.SetSubTitle("EI=70[meV]")
#    p.SliceXRange(-5,1,1)
#    p.SetWithPlot(True)
#    p.SliceYRange(3,5,4,0.1)
#    p.SetWithPlot(False)
#    sleep(2)
#    p.SetWithPlot(False, 2)
#    p.SliceX(0.5,2.0, 0.2)
#    p.AddToPlot(1)
#    p.SliceY(3.0,0.0,1)
#    p.SliceXRange
#    sleep(1)
#    p.AddToPlot(3) 
#    p.ClearPlot(3)
#    p.SetXLabel("X")
#    p.SetYLabel("yy Test")
#    p.SetZLabel("Z Axis")
#    sleep(2)
#    p.SetLog(False)
#    sleep(2)
#    p.SetZScale(0.0, 15.0)
#    sleep(2)
#    p.SetLog(True)
#    sleep(2)
#    p.SetZScale()
#    sleep(2)
#    p.Close(1)
#    sleep(2)
    



#! /usr/bin/env python
import sys
import os
################################################################################
# Compile script for Manyo-lib release version
#    ver 20200214
#    written by Y. Inamura
#########################################################################
man_script="Usage :     %s <command> [options]\n"%(sys.argv[0])
man_script+="""
       command: [depend/man/make/install/clean]
       options: --debug --help --prefix --manyo-path
                --other-modules --numth --pyver
                --rev --without-depend --without-para
                --oldinstalltype
  -------------------------------------------------------
  depend  : depend script
  man     : script to make manual
  make    : make
  install : make install
  clean   : make clean

  --debug          : debug mode
  --help           : show this help
  
  --prefix=/hoge/hogege/hoge
                   : set directory to install
  --manyo-path=/hoge/hogege/hoge
                   : set directory to install with old structures
  
  --dist-name=Ubuntu
                   : Linux distribution name [RH6,RH7,Ubuntu]
  --other-modules=SAS,DNA
                   : required other modules if need(default Utsusemi)
  
  --numth=n        : set the number of multi-threading
  --pyver=n.m      : set python version
                     If not set pyver, the version of python executing this is used
  --rev=nnnn       : set revision number if need
  --without-depend : does not run depend script
  --without-para   : not use compile option of -j (default use -j)
  --oldinstalltype : install to old path (/opt/mlfsfot/

"""

#########################################################################
#   CXX            : compiler
#   PYVER          : version of python
#   NUM_OF_THREADS : number of threading
#   PREFIX         : path to install
#   OTHER_MANYO_PAC: Other Manyo Package names required by compile
#   ENV_FILE_NAME  : file name of shell script to set Manyo environment 
#   EXT_LIB_ROOT   : root path to external library required by Manyo-lib
#   DIST_NAME      : name of linux distribution
#########################################################################
CXX            = "g++"
PYVER          = "%s.%s"%(sys.version_info.major, sys.version_info.minor)
NUM_OF_THREADS = 8
PREFIX         = "/opt/mlfsoft/utsusemi"
OTHER_MANYO_PAC= ["Utsusemi"]
ENV_FILE_NAME  = "bashrc.manyo.release"
EXT_LIB_ROOT   = "/usr/local"
DIST_NAME      = ""

isDebug = False
isWithDepend = True
isCompPara = True
isNotManyoCore = True
utsusemi_rev = 0
isOldInstallType = False
if sys.version_info.major==2:
    isOldInstallType = True
distNameDic = ["RH6","RH7","Ubuntu"]
#############################################
# Check arguments
#############################################

do_command = ""

#print(sys.argv[2:])
unidentified_args = []
if (len(sys.argv)>1):
    used_args = sys.argv[1:]
    num_of_args = len( used_args )
    ind = 0
    while(True):
        if ind==num_of_args:
            break
        an_arg = used_args[ind]
        ind += 1
        
        if an_arg in ["depend","make","install","clean","man"]:
            do_command = an_arg
        elif an_arg=="--debug":
            isDebug = True
        elif an_arg=="--without-depend":
            isWithDepend = False
        elif an_arg=="--without-para":
            isCompPara = False
        elif an_arg.find("--rev=")==0:
            utsusemi_rev = int(an_arg[6:])
            print("utsusemi_rev = %d"%utsusemi_rev)
        elif an_arg.find("--pyver=")==0:
            PYVER = an_arg[8:].strip()
            print("PYVER= "+PYVER)
        elif an_arg.find("--numth=")==0:
            NUM_OF_THREADS=int(an_arg[8:])
            print("NUM_OF_THREADS= %d"%(NUM_OF_THREADS))
        elif an_arg.find("--prefix=")==0:
            PREFIX=an_arg[9:]
            isOldInstallType = False
            print("PREFIX="+PREFIX)
        elif an_arg.find("--manyo-path=")==0:
            PREFIX=an_arg[13:]
            isOldInstallType = True
            print("PREFIX="+PREFIX)
        elif an_arg.find("--other-modules=")==0:
            mod_arg = an_arg[16:]
            OTHER_MANYO_PAC = mod_arg.split(",")
            for a_mod in OTHER_MANYO_PAC:
                print("OTHER_MANYO_PAC= "+a_mod)
        elif an_arg.find("--help")==0:
            do_command=""
            break
        elif an_arg.find("--dist-name=")==0:
            DIST_NAME=an_arg[12:]
            if not DIST_NAME in distNameDic:
                print(" ERROR : --dist-name is invalid")
                sys.exit(-1)
        elif an_arg.find("--oldinstalltype")==0:
            isOldInstallType = True
        else:
            print("Unindentify argument = "+an_arg)
            unidentified_args.append( an_arg )

if (do_command=="") or len(unidentified_args)!=0:
    print(man_script)
    sys.exit(0)

if (do_command=="clean"):
    isWithDepend = False

if (do_command=="make"):
    do_command=""

if os.path.exists( os.path.join( os.getcwd(),"core" ) ):
    isNotManyoCore = False

#############################################
## Check compile target ( manyo-core or not )
#############################################
if isNotManyoCore:
    PACKAGE_NAME=""
    if os.path.exists( os.path.join( os.getcwd(),"Makefile" ) ):
        fo = open( os.path.join( os.getcwd(),"Makefile" ), "r" )
        ll = fo.readlines()
        fo.close()
        for l in ll:
            print(l)
            if l.find("PACKAGE_NAME") is not -1:
                PACKAGE_NAME=l.split("=")[1].strip()
                break
    if PACKAGE_NAME=="":
        print( "Makefile is invalid. (No PACKAGE_NAME)" )
        sys.exit(1)


#########################################################################
## execute make or make install 
#############################################
OTHER_MAN_PAC=["Manyo","MLF"]
for pac in OTHER_MANYO_PAC:
    OTHER_MAN_PAC.append( pac )

EXT_LIB_PATH = os.path.join( EXT_LIB_ROOT,"lib" )
EXT_INC_PATH = os.path.join( EXT_LIB_ROOT,"include" )

if DIST_NAME=="Ubuntu":
    DIST_PACK = "dist-packages"
else:
    DIST_PACK = "site-packages"

com = ""
if isOldInstallType:
    MANYO_PATH = PREFIX
    com += "export MANYO_PATH={};".format(MANYO_PATH)
    com += "export LIBRARY_PATH=${MANYO_PATH}/lib"
    com += ":{};".format(EXT_LIB_PATH)
    com += "export LD_LIBRARY_PATH=${LIBRARY_PATH};"
    com += "export C_INCLUDE_PATH=/usr/include/python{}:".format(PYVER)
    com += ":${MANYO_PATH}/include"+":{}".format(EXT_INC_PATH)
    for m_pac_name in OTHER_MAN_PAC:
        com += ":${MANYO_PATH}/include/"+"{}".format(m_pac_name)
else:
    MANYO_PATH = os.path.join( PREFIX,"lib","python{}".format(PYVER),DIST_PACK)
    com += "export MANYO_PATH={};".format(MANYO_PATH)
    com += "export PREFIX={};".format(PREFIX)
    com += "export LIBRARY_PATH=${MANYO_PATH}:"+"{};".format(EXT_LIB_PATH)
    com += "export LD_LIBRARY_PATH=${LIBRARY_PATH};"
    com += "export C_INCLUDE_PATH=/usr/include/python{}:".format(PYVER)
    com += ":${PREFIX}/include/Manyo"+":{}".format(EXT_INC_PATH)
    for m_pac_name in OTHER_MAN_PAC:
        com += ":${PREFIX}/include/Manyo/"+"{}".format(m_pac_name)


if DIST_NAME=="Ubuntu":  ## [inamura for Ubuntu 16.04 or later]
    com += ":/usr/include/hdf5/serial"
com += "; "

com += 'export INC_EXT="'
for m_pac_name in OTHER_MAN_PAC:
    com += "%s "%(m_pac_name)
com += '"; '
com += "export CPLUS_INCLUDE_PATH=${C_INCLUDE_PATH};"

if isNotManyoCore and isWithDepend:
    com += "./depend; "

if (do_command!="depend"):
    com += 'make %s '%(do_command)
    if isCompPara:
        com += ' -j '
    if DIST_NAME=="":
        DIST_NAME="Ubuntu"
    com += ' DIST_NAME=%s'%(DIST_NAME)
    if PYVER!="":
        com += ' CXX=%s PyVer=%s NUM_MULTH=%d' % (CXX,PYVER,NUM_OF_THREADS)
    else:
        com += ' CXX=%s NUM_MULTH=%d' % (CXX,NUM_OF_THREADS)
    com += ' UTSUSEMI_REV=%d' % (utsusemi_rev)
    if not isOldInstallType and PYVER!="":
        com += " PY_PREFIX={}".format(MANYO_PATH)

print("command = "+com)
if isDebug:
    pass
else:
    retcode = os.system(com)
    if retcode:
        sys.stderr.write("Return code '{}', is not 0.\n".format(retcode))
        raise RuntimeError("Error has been caused in building.")



#########################################################################
## make environ file
#############################################

env_list = []
env_list.append( '#!/bin/bash' )
env_list.append( 'export MANYO_PATH=%s' % (MANYO_PATH) )

if isOldInstallType:
    env_list.append( 'export LD_LIBRARY_PATH=${MANYO_PATH}/lib:${LD_LIBRARY_PATH}' )
else:
    env_list.append( 'export LD_LIBRARY_PATH=${MANYO_PATH}:${LD_LIBRARY_PATH}' )

if isNotManyoCore:
    env_list.append( 'export PYTHONPATH=${MANYO_PATH}:${MANYO_PATH}/Manyo/%s:${PYTHONPATH}' % (PACKAGE_NAME) )
else:
    env_list.append( 'export PYTHONPATH=${MANYO_PATH}:${PYTHONPATH}' )
env_list.append( 'export PATH={}/bin:$PATH'.format(PREFIX) )


if isDebug:
    for an_env in env_list:
        print( an_env )
else:
    fo = open( os.path.join( os.getcwd(),ENV_FILE_NAME ), 'w' )
    for ss in env_list:
        fo.write(ss+'\n')
    fo.close()





# -*- coding: utf-8 -*-

import os
import sys
import wx
import wx.xrc as xrc
import ana.Reduction.D4Mat_Manyo as D4Mat
import vis.M2Plot as M2P
import vis.D2Vis as D2Vis
import vis.UtilPlot as UtilPlot
import threading
import time
import vis.VisContMParams as VCP
import Manyo.Utsusemi as mu
import Manyo as m


################################################
#                D4MatSlicer
#    GUI program to treat D4Matrix on Manyo-lib
#          Y.Inamura
#(2016.06.21-23) ver05 Improve SliceMat and add VirtualD4Mat
#(2016.05.23) ver041 Improve OutputText3D to change treating MASKVALUE
#(2015.04.16) ver04 Add OutputText3D 
#(2011.05.02) ver03 Revise treating axes titles (requred Manyo-lib >2250)
#(2011.04.28) ver02 Add Rebin func.
#(2011.04.11) ver01 firat version
################################################
__version__ = 'ver05(20160623)'

############################
## Load XMl resources file
############################
__XRCRESOURCEFILE = os.path.join( os.environ["UTSUSEMI_BASE_DIR"],'vis','D4MatSlicer.xrc' )
#__XRCRESOURCEFILE = 'D4MatGUI.xrc'
__res = None

def get_resources():
    """ This function provides access to the XML resources in this module."""
    global __res
    if __res == None:
        __init_resources()
    return __res

def __init_resources():
    global __res
    __res = xrc.EmptyXmlResource()

    __res.Load(__XRCRESOURCEFILE)

###################################
## Make New Matrix Window
###################################
class MakeNewMatrixFrame(wx.Frame):
    def __init__(self,parent):
        """
        """
        self.parent = parent
        
        self.frame = get_resources().LoadFrame(parent.frame, 'NewD4MatrixFrame')
        self.frame.Bind( wx.EVT_BUTTON, self.OnLoadParams, id=xrc.XRCID("btLoadParams"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnMakeNewMatrix, id=xrc.XRCID("btMakeNewMatrix"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnClose, id=xrc.XRCID("btClose"))
        
        mpanel = xrc.XRCCTRL( self.frame, "NewD4MatrixPanel" )
        self.D4MatRangeCtrl = []
        self.D4MatTitle = []
        for i in range(12):
            self.D4MatRangeCtrl.append( xrc.XRCCTRL( mpanel, ( "txt%d" % (i+1) ) ) )
            
        for i in range(4):
            self.D4MatTitle.append( xrc.XRCCTRL( mpanel, ( "v%dtitle" % (i+1) ) ) )

        self.frame.Bind(wx.EVT_CLOSE, self.OnClose)
        self.CommentHead = "D4MatGUI::MakeNewMatrix >>> "
        
        # Justify the alignment of controls on OSX and Windows
        if sys.platform == "darwin":
            self.frame.SetSizeWH( 460,200 )
        if sys.platform=="win32":
            self.frame.SetSizeWH( 455,220 )
        
        # show window
        self.frame.Show()

    #################################
    def OnLoadParams(self,evt):
        """
        """
        print self.CommentHead+"OnLoadParams"
        # open dialog
        dlg = wx.FileDialog(self.frame, 'Open VisCont Parameter file ...', os.getcwd(), style=wx.OPEN, wildcard='XML(*.xml)|*.xml')

        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return

        # get file path
        filepath = dlg.GetPath()
        dir = os.path.dirname(filepath)

        (ax_list,title_list) = self.parent.D4Mat.loadVisContParams(filepath)
        for i in range(4):
            ax = ax_list[i]
            self.D4MatRangeCtrl[ i*3 ].SetValue( str("%g" % ax[0]) )
            self.D4MatRangeCtrl[ i*3 + 1 ].SetValue( str("%g" % ax[1]) )
            self.D4MatRangeCtrl[ i*3 + 2 ].SetValue( str("%g" % ax[2]) )
            self.D4MatTitle[ i ].SetValue( title_list[i] )
        
        return

    #################################
    def OnMakeNewMatrix(self,evt):
        """
        """
        print self.CommentHead+"OnMakeNewMatrix"

        # open directory dialog
        dlg = wx.FileDialog( self.frame, 'Choose File to save D4Matrix parameters ...', defaultDir=os.getcwd(), defaultFile='D4Mat.xml', wildcard='xml File(*.xml)|*.xml',style=wx.SAVE )

        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return

        # get path
        filepath = dlg.GetPath()
        dir = os.path.dirname(filepath)
        #print "filepath=",filepath
        #print "dir=",dir
        
        ax_list=[]
        title_list=[]
        for ax in range(4):
            ra = []
            for i in range(3):
                ra.append( float( self.D4MatRangeCtrl[ ax*3 + i ].GetValue() ) )
            ax_list.append(ra)
            title_list.append( self.D4MatTitle[ax].GetValue() )

        pre_label = self.parent.StatusTextCtrl.GetLabel()
        self.parent.StatusTextCtrl.SetLabel( "Making New D4Matrix files..." )
        self.parent.frame.Refresh()
        self.parent.frame.Update()
        self.frame.Refresh()
        self.frame.Update()
        
        self.parent.D4Mat.new( ax_list[0], ax_list[1], ax_list[2], ax_list[3], title_list, str(filepath) )
        ##self.parent.D4Mat.D4Mat._MakeNewParamXml( str(filepath)+".xml" ) ##[inamura 110411]
        for ax in ax_list:
            print self.CommentHead,ax
        
        self.parent.D4Mat.close()
        flag,msg = self.parent._OpenD4Matrix(str(filepath))
        if not flag:
            dial = wx.MessageDialog(None, msg, "Warning", wx.OK | wx.ICON_EXCLAMATION )
            dial.ShowModal()
            return

        self.parent.StatusTextCtrl.SetLabel( pre_label )

        dlg = wx.MessageDialog(self.frame, "Making New D4Matrix files completed.","",wx.OK)
        ret = dlg.ShowModal()
        
        return

    ##########################################################
    def OnClose(self, evt):
        """
        Close button event
        @param  evt event information
        @retval None
        """        
        self.parent.StatusTextCtrl.SetLabel( "Done." )
        self.frame.Destroy()
        

###################################
## Rebin Matrix Window
###################################
class RebinMatrixFrame(wx.Frame):
    def __init__(self,parent):
        """
        """
        self.parent = parent

        if not self.parent.D4Mat.isOpened():
            dlg = wx.MessageDialog(self.parent.frame, "You need to open D4Matrix at first!!","Warning",wx.OK)
            ret = dlg.ShowModal()
            return
        
        self.frame = get_resources().LoadFrame(parent.frame, 'RebinD4MatrixFrame')
        self.frame.Bind( wx.EVT_BUTTON, self.OnRebinMatrix, id=xrc.XRCID("btRebinMatrix"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnClose, id=xrc.XRCID("btClose"))
        
        mpanel = xrc.XRCCTRL( self.frame, "RebinD4MatrixPanel" )
        self.D4MatRangeCtrl = []
        self.D4MatTitle = []
        self.RebinRatio = []
        for i in range(12):
            self.D4MatRangeCtrl.append( xrc.XRCCTRL( mpanel, ( "txt%d" % (i+1) ) ) )
            
        for i in range(4):
            self.RebinRatio.append( xrc.XRCCTRL( mpanel, ( "spn%dStepRatio" % (i+1) ) ) )
            self.D4MatTitle.append( xrc.XRCCTRL( mpanel, ( "v%dtitle" % (i+1) ) ) )

        self.frame.Bind(wx.EVT_CLOSE, self.OnClose)

        for i in range(4):
            self.D4MatRangeCtrl[ i*3 + 0 ].SetLabel( self.parent.D4MatRangeCtrl[ i*3 + 0 ].GetLabel() )
            self.D4MatRangeCtrl[ i*3 + 1 ].SetLabel( self.parent.D4MatRangeCtrl[ i*3 + 1 ].GetLabel() )
            self.D4MatRangeCtrl[ i*3 + 2 ].SetLabel( self.parent.D4MatRangeCtrl[ i*3 + 2 ].GetLabel() )
            self.D4MatTitle[ i ].SetValue( self.parent.D4MatTitle[ i ].GetValue() )

        self.CommentHead = "D4MatGUI::RebinMatrix >>> "
        
        # Justify the alignment of controls on OSX and Windows
        if sys.platform == "darwin":
            self.frame.SetSizeWH( 510,220 )
        if sys.platform=="win32":
            self.frame.SetSizeWH( 510,220 )
        
        # show window
        self.frame.Show()

    #################################
    def OnRebinMatrix(self,evt):
        """
        """
        print self.CommentHead+"OnRebinMatrix"

        # open directory dialog
        dlg = wx.FileDialog( self.frame, 'Choose File to save D4Matrix parameters ...', defaultDir=os.getcwd(), defaultFile='D4Mat.xml', wildcard='xml File(*.xml)|*.xml',style=wx.SAVE )

        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return

        # get path
        filepath2 = dlg.GetPath()
        
        # get path2
        ( dir1, filename1 ) = self.parent.D4Mat.putOpenedPath()
        filepath1 = os.path.join( dir1,filename1 )
        
        ax_list=[]
        title_list=[]
        ratio_list=[]
        for ax in range(4):
            ra = []
            for i in range(3):
                ra.append( float( self.D4MatRangeCtrl[ ax*3 + i ].GetLabel() ) )
            ratio_list.append( float( self.RebinRatio[ax].GetValue() ) )
            print self.CommentHead+"Ratio="+str( ratio_list[-1] )
            ra[2] = ra[2]*ratio_list[-1]
            ax_list.append(ra)
            title_list.append( self.D4MatTitle[ax].GetValue() )

        for ax in ax_list:
            print self.CommentHead,ax
            
        pre_label = self.parent.StatusTextCtrl.GetLabel()
        self.parent.StatusTextCtrl.SetLabel( "Making New D4Matrix files..." )
        self.parent.frame.Refresh()
        self.parent.frame.Update()
        self.frame.Refresh()
        self.frame.Update()
        
        self.parent.D4Mat.new( ax_list[0], ax_list[1], ax_list[2], ax_list[3], title_list, str(filepath2) )
        self.parent.D4Mat.close()

        self.parent.StatusTextCtrl.SetLabel( "Rebinning D4Matrix ..." )
        self.parent.frame.Refresh()
        self.parent.frame.Update()
        self.frame.Refresh()
        self.frame.Update()
        
        self.parent.D4Mat.rebin( filepath1, filepath2 )        
        
        flag,msg = self.parent._OpenD4Matrix(str(filepath2))
        if not flag:
            dial = wx.MessageDialog(None, msg, "Warning", wx.OK | wx.ICON_EXCLAMATION )
            dial.ShowModal()
            return

        self.parent.StatusTextCtrl.SetLabel( pre_label )

        dlg = wx.MessageDialog(self.frame, "Rebinning D4Matrix completed.","",wx.OK)
        ret = dlg.ShowModal()
        
        return
        
        
    ##########################################################
    def OnClose(self, evt):
        """
        Close button event
        @param  evt event information
        @retval None
        """ 
        self.parent.StatusTextCtrl.SetLabel( "Done." )
        self.frame.Destroy()

##[inamura 160523]-->
#########################################
#       SetSaveAsTextDialog
#########################################
class SetSaveAsTextDialog(wx.Dialog):
    """
    SaveAsText 設定ダイアログクラス
    """
    ##########################################################
    def __init__(self, parent, frame):
        """
        コンストラクタ
        @param  parent   親ウィンドウ
        @param  dataobj
        @retval 無し
        """ 
        
        self.parent = parent
        self.frame = frame
        
        # リソース取得      
        self.dialog = get_resources().LoadDialog(frame,"Dg2DSaveAsText")
        
        # チェックボックス
        self.ckIgnoreMask = xrc.XRCCTRL(self.dialog,'ckIgnoreMask')
        
        # チェックボックスのイベントハンドラ登録
        self.ckIgnoreMask.Bind(wx.EVT_CHECKBOX, self.OnIgnoreMask, self.ckIgnoreMask)
        
        # コンボボックス
        self.chMaskVal = xrc.XRCCTRL(self.dialog, 'chMaskValue')
        
        # コンボボックスのイベントハンドラ登録
        self.chMaskVal.Bind(wx.EVT_CHOICE, self.OnMaskVal, self.chMaskVal)
        
        # テキストボックスのコントローラ取得        
        self.txMaskVal = xrc.XRCCTRL(self.dialog, 'txMaskValue')
        
        # ボタンのイベントハンドラを登録
        self.btnSave = xrc.XRCCTRL( self.dialog, 'btSave' )
        self.btnCancel = xrc.XRCCTRL( self.dialog, 'btCancel' )
        self.btnSave.Bind(wx.EVT_BUTTON, self.OnSave, self.btnSave )
        self.btnCancel.Bind(wx.EVT_BUTTON, self.OnCancel, self.btnCancel )
        
        #[X]ボタン押下時のイベントハンドラ登録
        self.dialog.Bind(wx.EVT_CLOSE, self.OnCancel)
        
        # Set Values
        self.initialize()
        
        # ダイアログ表示
        self.dialog.Show()
        
    ##########################################################
    def initialize(self):
        """
        """
        self.isIgnoreMask = True
        self.useGivenMaskVal = True
        self.ckIgnoreMask.SetValue(True)
        self.chMaskVal.Enable(False)
        self.txMaskVal.Enable(False)
        return
        
    ##########################################################
    def OnIgnoreMask(self, event=None):
        """
        IgnoreMask チェックボックスイベント処理
        @param   event イベント情報   
        @retval 無し
        """ 
        # チェックされたのか
        self.isIgnoreMask = self.ckIgnoreMask.GetValue()
        if self.isIgnoreMask:
            # MaskValue テキストボックスを選択不可
            self.chMaskVal.Enable(False)
            self.txMaskVal.Enable(False)
        else:  
            self.chMaskVal.Enable(True)
            self.txMaskVal.Enable(False)
            index = self.chMaskVal.GetSelection()
            if index==0:
                self.txMaskVal.Enable(True)
                
    ##########################################################
    def OnMaskVal(self, event=None):
        """
        
        """
        self._CtrlParts(True)
        
    ##########################################################
    def OnSave(self, event=None):
        """
        """
        self._CtrlParts(False)
        maskval = 0.0
        if (not self.isIgnoreMask) and self.useGivenMaskVal:
            try:
                maskval = float( self.txMaskVal.GetValue() )
            except:
                msg = "Given Mask Value is invalid (%s)"%(self.txMaskVal.GetValue())
                dial = wx.MessageDialog(None, msg, "Warning", wx.OK | wx.ICON_EXCLAMATION )
                dial.ShowModal()
                self._CtrlParts(True)
                return
        

        maskValStr = "MASKVAL"
        if (self.useGivenMaskVal):
            maskValStr = self.txMaskVal.GetValue()
        
        # open dialog
        dlg = wx.FileDialog(self.frame, 'Choose 3D text output file ...', os.getcwd(),
                            style=wx.SAVE, wildcard='TEXT(*.txt)|*.txt')
        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return
        
        #filepath = str( dlg.GetPath() )
        filepath = str( dlg.GetFilename() )
        self.parent.StatusTextCtrl.SetLabel( "Out3D saves file %s" % (filepath ) )
        self.parent.frame.Refresh()
        self.parent.frame.Update()
        
        Axr = []
        type_axes = []
        foldings = []
        for i in range(4):
            #ax = self.parent.SliceAxis[i].GetSelection() #[inamura 160616]
            ax = self.parent.SliceAxis[i].GetCurrentSelection()
            if ax == 0:
                type_axes.append("X")
            elif ax == 1:
                type_axes.append("Y")
            elif ax == 2:
                type_axes.append("Z")  ##[inamura 160616]
            elif ax == 3:
                type_axes.append("T")  ##[inamura 160616]
            else:
                raise
            
            ##[inamura 160616]-->
            #if self.parent.SliceFoldCB[i].GetValue():
            #    foldings.append(1)
            #else:
            #    foldings.append(0)
            foldings.append( float( self.parent.SliceFoldCB[i].GetValue() ) )
            ##<--[inamura 160616]
            ar = []
            ar.append( float( self.parent.SliceRange[i*3].GetValue() ) )
            ar.append( float( self.parent.SliceRange[i*3+1].GetValue() ) )
            Axr.append(ar)
            
        import Manyo as mm
        a1v = mm.ListToDoubleVector( Axr[0] )
        a2v = mm.ListToDoubleVector( Axr[1] )
        a3v = mm.ListToDoubleVector( Axr[2] )
        a4v = mm.ListToDoubleVector( Axr[3] )
        def_axes = mm.MakeStringVector()
        #fld = mm.ListToInt4Vector( foldings )  ##[inamura 160616]
        fld = mm.ListToDoubleVector( foldings )
        for Axtype in type_axes:
            def_axes.append(Axtype)


        self.parent.D4Mat.D4Mat.OutputText3D(a1v, a2v, a3v, a4v, def_axes, fld, str(filepath), self.isIgnoreMask, str(maskValStr) )
        self.parent.StatusTextCtrl.SetLabel( "Ready" )

        msg = "Finished Save 3D data to text file : "+filepath
        dial = wx.MessageDialog(None, msg, "Message", wx.OK | wx.ICON_INFORMATION )
        dial.ShowModal()
        self._CtrlParts(True)
        self.OnCancel()

    ##########################################################
    def _CtrlParts(self, isEnable=True):
        """
        """
        if not isEnable in [True,False]:
            return
            
        if isEnable:
            self.btnSave.Enable(True)
            self.btnCancel.Enable(True)
            self.ckIgnoreMask.Enable(True)
            if self.isIgnoreMask:
                self.chMaskVal.Enable(False)
                self.txMaskVal.Enable(False)
            else:
                self.chMaskVal.Enable(True)
                self.txMaskVal.Enable(False)
                index = self.chMaskVal.GetSelection()
                self.useGivenMaskVal = True
                if index==0:
                    self.txMaskVal.Enable(True)
                    pass
                elif index==1:
                    self.txMaskVal.SetValue( str(0.0) )
                elif index==2:
                    self.txMaskVal.SetValue( str(-1.0) )
                else:
                    self.useGivenMaskVal = False
                    self.txMaskVal.SetValue( "" )
        else:
            self.ckIgnoreMask.Enable(False)
            self.chMaskVal.Enable(False)
            self.txMaskVal.Enable(False)
            self.btnSave.Enable(False)
            self.btnCancel.Enable(False)
            
            
    ##########################################################
    def OnCancel(self, evt=None):
        """
        Cancel ボタンイベント処理
        @param  evt イベント情報
        @retval 無し
        """ 
        
        self.dialog.Destroy()        

#[inamrua 160621]-->
###################################
## Make Virtual Matrix Window
###################################
class MakeVirtualD4MatrixFrame(wx.Frame):
    def __init__(self,parent):
        """
        """
        self.parent = parent
        
        self.frame = get_resources().LoadFrame(parent.frame, 'VirtualD4MatrixFrame')
        self.frame.Bind( wx.EVT_BUTTON, self.OnLoadParams, id=xrc.XRCID("btLoadParams"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnSaveParams, id=xrc.XRCID("btSaveParams"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnFillRangeVirtualMatrix, id=xrc.XRCID("btFillRange"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnMakeVirtualMatrix, id=xrc.XRCID("btMakeVirtualMatrix"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnClear, id=xrc.XRCID("btClr"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnMakeNewD4Mat, id=xrc.XRCID("btMakeNewD4Mat"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnEstimateSize, id=xrc.XRCID("btEstimateD4MatSize"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnClose, id=xrc.XRCID("btClose"))
        self.BtnCtrls=[]
        self.BtnCtrls.append( xrc.XRCCTRL( self.frame, "btLoadParams" ) )
        self.BtnCtrls.append( xrc.XRCCTRL( self.frame, "btSaveParams" ) )
        self.BtnCtrls.append( xrc.XRCCTRL( self.frame, "btFillRange" ) )
        self.BtnCtrls.append( xrc.XRCCTRL( self.frame, "btMakeVirtualMatrix" ) )
        self.BtnCtrls.append( xrc.XRCCTRL( self.frame, "btClr" ) )
        self.BtnCtrls.append( xrc.XRCCTRL( self.frame, "btMakeNewD4Mat") )
        self.BtnCtrls.append( xrc.XRCCTRL( self.frame, "btEstimateD4MatSize" ) )
        self.BtnCtrls.append( xrc.XRCCTRL( self.frame, "btClose" ) )
        
        mpanel = xrc.XRCCTRL( self.frame, "VirtualD4MatrixPanel" )
        mpanel0 = xrc.XRCCTRL( self.frame, "panel0" )
        mpanel1 = xrc.XRCCTRL( self.frame, "panel1" )
        mpanel2 = xrc.XRCCTRL( self.frame, 'panel2')
        mpanel3 = xrc.XRCCTRL( self.frame, 'panel3')
        mpanel4 = xrc.XRCCTRL( self.frame, 'panel4')
        
        self.minfo = []
        self.minfo.append( xrc.XRCCTRL(mpanel0,"txEi") )
        self.minfo.append( xrc.XRCCTRL(mpanel0,"txminhw") )
        self.minfo.append( xrc.XRCCTRL(mpanel0,"txmaxhw") )
        self.minfo.append( xrc.XRCCTRL(mpanel0,"txdeltahw") )
        self.minfo.append( xrc.XRCCTRL(mpanel0,"txminphi") )
        self.minfo.append( xrc.XRCCTRL(mpanel0,"txmaxphi") )
        self.minfo.append( xrc.XRCCTRL(mpanel0,"txdeltaphi") )
        
        self.runnotxt = xrc.XRCCTRL(mpanel0, 'txrunno' )
        
        # サンプル情報テキストボックスのコントロールを取得
        self.sinfo = []
        for i in range(15):
            ii = i+1
            txt = "txt%d" % ii
            self.sinfo.append(xrc.XRCCTRL(mpanel1, txt)) 

        self.chrotax1 = xrc.XRCCTRL(mpanel1, 'chRotateAxis1st') 
        self.chrotax2 = xrc.XRCCTRL(mpanel1, 'chRotateAxis2nd')
        self.chrotax3 = xrc.XRCCTRL(mpanel1, 'chRotateAxis3rd')            
        self.chRotationSteps = [self.chrotax1,self.chrotax2,self.chrotax3]


        self.chrotax1.Bind(wx.EVT_CHOICE, self.OnChoiceRotateAxes, self.chrotax1) 
        self.chrotax2.Bind(wx.EVT_CHOICE, self.OnChoiceRotateAxes, self.chrotax2) 
        self.chrotax3.Bind(wx.EVT_CHOICE, self.OnChoiceRotateAxes, self.chrotax3) 
        
        # Projectionテキストボックスのコントロールを取得

        self.uv = []
        for i in range(4):
            uv0 = []
            for j in range(5):
               ii = i*5+j+1
               txt = "ptxt%d" % ii
               uv0.append( xrc.XRCCTRL(mpanel2, txt)) 
            self.uv.append(uv0)



        self.D4MatTitle = []
        for i in range(4):
            self.D4MatTitle.append( xrc.XRCCTRL( mpanel2, ( "ptxt%d"%( i*5 + 5 ) ) ) )
            
        self.D4MatRangeCtrl = []        
        for i in range(12):
            self.D4MatRangeCtrl.append( xrc.XRCCTRL( mpanel4, ( "txt%d" % (i+1) ) ) )
           
        self.frame.Bind(wx.EVT_CLOSE, self.OnClose)
        self.CommentHead = "D4MatGUI::MakeVirtualMat >>> "
        
        menubar = get_resources().LoadMenuBar("MakeVirtualMenuBar")
        self.frame.SetMenuBar( menubar )
        self.frame.Bind(wx.EVT_MENU, self.OnLoadParams, id=xrc.XRCID('menu_load_params'))
        self.frame.Bind(wx.EVT_MENU, self.OnSaveParams, id=xrc.XRCID('menu_save_params'))
        self.frame.Bind(wx.EVT_MENU, self.OnClose, id=xrc.XRCID('menu_exit'))

        # Justify the alignment of controls on OSX and Windows
        if sys.platform == "darwin":
            self.frame.SetSizeWH( 510, 750 )
        if sys.platform=="win32":
            self.frame.SetSizeWH( 490, 755 )

        # show window
        self.frame.Show()

    #################################
    def OnLoadParams(self,evt):
        """
        """
        print self.CommentHead+"OnLoadParams"
        
        # open dialog
        dlg = wx.FileDialog(self.frame, 'Open VisCont Parameter file ...', os.getcwd(), style=wx.OPEN, wildcard='XML(*.xml)|*.xml')

        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return

        # get file path
        filepath = dlg.GetPath()
        dir = os.path.dirname(filepath)

        if not os.path.exists(filepath):
            msg = " Cannot find XML file(%s)" % (filepath)
            print self.CommentHead+"::OnLoadParams >>"+msg
            dial = wx.MessageDialog(None, msg, "Warning", wx.OK | wx.ICON_EXCLAMATION )
            dial.ShowModal()
            return

        vcp = VCP.VisContMParams(filepath)
        try:
            lc=vcp.Header['SimpleLatticeConsts']
            uv=vcp.Header['SampleUvec']
            vv=vcp.Header['SampleVvec']
        except:
            msg = "File is invalid. ("+filepath+")"
            print self.CommentHead+"::OnLoadParams >>"+msg
            dial = wx.MessageDialog(None, msg, "Warning", wx.OK | wx.ICON_EXCLAMATION )
            dial.ShowModal()
            return

            
        self.sinfo[0].SetValue( "%6.4f" % lc[0] )
        self.sinfo[1].SetValue( "%6.4f" % lc[1] )
        self.sinfo[2].SetValue( "%6.4f" % lc[2] )
        self.sinfo[3].SetValue( "%6.4f" % lc[3] )
        self.sinfo[4].SetValue( "%6.4f" % lc[4] )
        self.sinfo[5].SetValue( "%6.4f" % lc[5] )
        self.sinfo[6].SetValue( "%6.4f" % uv[0] )
        self.sinfo[7].SetValue( "%6.4f" % uv[1] )
        self.sinfo[8].SetValue( "%6.4f" % uv[2] )
        self.sinfo[9].SetValue( "%6.4f" % vv[0] )
        self.sinfo[10].SetValue( "%6.4f" % vv[1] )
        self.sinfo[11].SetValue( "%6.4f" % vv[2] )
        
        ro_list = vcp.Header["SampleRotateSteps"]
        ro_axis_dic={ "Y":1,"Z":2,"X":3 }
        for i,a_rot in enumerate(ro_list):
            ax_num=0
            try:
                ax_num = ro_axis_dic[ a_rot[0] ]
            except:
                pass
            self.chRotationSteps[i].Enable(True)
            self.chRotationSteps[i].SetSelection( ax_num )
            self.sinfo[12+i].SetValue( "%6.4f" % float(a_rot[1]) )
            self.sinfo[12+i].Enable(True)
            if (i+1)<len(self.chRotationSteps):
                self.chRotationSteps[i+1].Enable(True)
            
        ## Projection infor
        for id in range(4):
            for col in range(4):
                self.uv[id][col].SetValue( "%g" % (vcp.proj_axes[str(id)][col]) )
            self.uv[id][4].SetValue( vcp.proj_axes[str(id)][4] )
            
        ## D4Matrix range
        for id in range(4):
            range_axis = vcp.paxis_info[str(id)][1]
            self.D4MatRangeCtrl[3*id + 0].SetValue( "%g" % ( range_axis[0] ) )
            self.D4MatRangeCtrl[3*id + 1].SetValue( "%g" % ( range_axis[1] ) )
            self.D4MatRangeCtrl[3*id + 2].SetValue( "%g" % ( range_axis[2] ) )
            self.D4MatTitle[ id ].SetValue( vcp.proj_axes[str(id)][4] )

        return

    #################################
    def OnSaveParams(self,evt):
        """
        """
        print self.CommentHead+"OnSaveParams"
        
        sample=[]
        for i in range(15):
            anum = self.sinfo[i].GetValue().strip()
            try:
                num = float(anum)
            except:
                raise UtilPlot.PlotException('Common', 'C027', ("Sample Info",))
            sample.append(num)
        
        Header = {}
        Header['SimpleLatticeConsts']=sample[:6]
        Header['SampleUvec']=sample[6:9]
        Header['SampleVvec']=sample[9:12]
        ##[inamura 150527]-->
        #Header['SampleRotatePhi']=sample[12]
        rotationSteps=[]
        for i,rax in zip([0,1,2],self.chRotationSteps):
            if rax.IsEnabled():
                ax_num = rax.GetCurrentSelection()
                if ax_num == 1:
                    rotationSteps.append( ["Y", sample[12+i] ] )
                elif ax_num == 2:
                    rotationSteps.append( ["Z", sample[12+i] ] )
                elif ax_num == 3:
                    rotationSteps.append( ["X", sample[12+i] ] )
                else:
                    break
        Header['SampleRotateSteps']=rotationSteps
        ##<--[inamura 150527]
        
        # Make parameters to give to VisContParams
        ## axisInfo
        axisInfo = []
        for id in range(4):
            a_list = []
            for col in range(4):
                a_list.append( float( self.uv[id][col].GetValue() ) )
            a_list.append( self.uv[id][4].GetValue() )
            axisInfo.append(a_list)

        ## sliceInfo
        type_list = ["x","x","x","x"]

        fold_list = [-1.0,-1.0,-1.0,-1.0]
        
        sliceInfo = []
        for id in range(4):
            a_list = [type_list[id]]
            for i in range(3):
                a_list.append( float( self.D4MatRangeCtrl[3*id + i].GetValue() ) )
            a_list.append( fold_list[id] )
            sliceInfo.append( a_list )

        # make instance
        vcp=VCP.VisContMParams()

        # set parameters
        vcp.SetAllInfo( Header, axisInfo, sliceInfo, [] ) #[inamura 1307xx]

        # open dialog
        dlg = wx.FileDialog(self.frame, 'Save Parameter File  ...', os.getcwd(), style=wx.SAVE, wildcard='XML(*.xml)|*.xml')

        # if cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return

        # get file path
        savefile = dlg.GetPath()
        dir = os.path.dirname(savefile)

        # show parameters to be checked
        #vcp.ShowParams()

        # save to XML file
        vcp.SaveXML( savefile )

    #################################
    def OnFillRangeVirtualMatrix(self,evt):
        """
        """
        print self.CommentHead+"OnFillRangeVirtualMatrix"
        runNo = int(self.runnotxt.GetValue())
        UGH = mu.UtsusemiGetNeunetHistogram()
        if UGH.SetRunNo( str(runNo) ):
            pass
        else:
            raise UserWarning, "runNo is invalid"
        UGH.SetDetParam("psd")
        
        Ei = float( self.minfo[0].GetValue() )
        minhw = float( self.minfo[1].GetValue() )
        maxhw = float( self.minfo[2].GetValue() )
        deltahw = float( self.minfo[3].GetValue() )
        if deltahw<=0.0 or minhw>maxhw or maxhw>=Ei :
            msg = "Energy Arguments are invalid."
            dial = wx.MessageDialog(None, msg, "Warning", wx.OK | wx.ICON_EXCLAMATION )
            dial.ShowModal()
            return
            
        minPhi = float( self.minfo[4].GetValue() )
        maxPhi = float( self.minfo[5].GetValue() )
        delPhi = float( self.minfo[6].GetValue() )
        if delPhi==0.0:
            delPhi=1.0;
            self.minfo[6].SetValue( str(delPhi) )
            
        hist_param = "hw,%f,%f,%f,%f"%(Ei,minhw,maxhw,deltahw)
        if UGH.SetConversionParameter( hist_param ):
            pass
        else:
            raise UserWarning, "hist param (%s) is invalid"%(hist_param)
        
        pfiles = UGH.MakeTempInfoFiles()
        if pfiles.size()<2:
            raise UserWarning, "fails to make tmp parameter files."
        
        btn_status = []
        for abtn in self.BtnCtrls:
            btn_status.append( abtn.IsEnabled() )
            abtn.Disable()
        self.frame.Update()

        lc = m.MakeDoubleVector(6)
        for i in range(6):
            lc[i] = float(self.sinfo[i].GetValue())

        uv = m.MakeDoubleVector(3)
        vv = m.MakeDoubleVector(3)
        for i in range(3):
            uv[i] = float(self.sinfo[6+i].GetValue())
            vv[i] = float(self.sinfo[9+i].GetValue())

        rotSteps = m.MakeDoubleVector()
        ## self.chRotationSteps is [self.chrotax1,self.chrotax2,self.chrotax3]
        for i in range(3):
            if self.sinfo[12+i].IsEnabled():
                axis_num = -1.0
                if self.chRotationSteps[i].GetCurrentSelection()==1:   # if Y-axis
                    axis_num = 1.0                             # 1 means Y-axis in VisualCalcSqe
                elif self.chRotationSteps[i].GetCurrentSelection()==2: # if Z-axis
                    axis_num = 2.0                             # 2 means Z-axis in VisualCalcSqe
                elif self.chRotationSteps[i].GetCurrentSelection()==3: # if X-axis
                    axis_num = 0.0                             # 0 means X-axis in VisualCalcSqe
                else:
                    break
                if axis_num > -1.0:
                    rotSteps.append( axis_num )                # axis number
                    rotSteps.append( float( self.sinfo[12+i].GetValue() ) )   # angle
            else:
                break

        va = m.MakeDoubleVector(16)
        titles = m.MakeStringVector(4)
        for id in range(4):
            for col in range(4):
                va[id*4 + col] = float( self.uv[id][col].GetValue() )
            print "id=",id,self.uv[id][4].GetValue()
            titles[id]= str( self.uv[id][4].GetValue() )

        hw_info = m.MakeDoubleVector(4)
        hw_info[0] = Ei
        hw_info[1] = deltahw
        hw_info[2] = minhw
        hw_info[3] = maxhw
        
        if minPhi>maxPhi:
            tmp_phi = minPhi
            minPhi = maxPhi
            maxPhi = tmp_phi
        if delPhi<0:
            delPhi = -1.0*delPhi
        
        phi_steps = m.MakeDoubleVector()
        phi = minPhi
        while( phi<maxPhi ):
            phi_steps.push_back( phi )
            phi += delPhi
        
        stime = time.time()        
        self.parent.D4Mat.D4Mat._CalcVirtualAngleInfo( pfiles[0], pfiles[1] )
        est_ranges=self.parent.D4Mat.D4Mat.EstimateRangeOfVirtualMat(lc, uv, vv, rotSteps, va, hw_info, phi_steps)
        print "  Allocation is finish : cost = ",time.time()-stime

        for i in range(4):
            self.D4MatRangeCtrl[ i*3 ].SetValue("%5.2f"%(est_ranges[i*2]))
            self.D4MatRangeCtrl[ i*3 + 1 ].SetValue("%5.2f"%(est_ranges[i*2+1]))
        self.D4MatRangeCtrl[ 11 ].SetValue("%5.2f"%(deltahw))
        
        self.parent.FileListBox.Clear()
        
        for btn_ctrl in self.parent.ButtonCtrlList:
            btn_ctrl.Enable(True)
        self.parent.isEnableSlice=True       #[inamura 170228]
        self.parent.CheckEnableSliceButton() #[inamura 170228]

        #for i in [4,5,6,9,10,11]:
        for i in [4,5,6,8,9,10]:
            self.parent.ButtonCtrlList[i].Enable(False)
            
        for astatus,abtn in zip(btn_status,self.BtnCtrls):
            abtn.Enable(astatus)
        self.frame.Update()
        
        return
        
    #################################
    def OnMakeVirtualMatrix(self,evt):
        """
        """
        print self.CommentHead+"OnMakeVirtualMatrix"
        runNo = int(self.runnotxt.GetValue())
        UGH = mu.UtsusemiGetNeunetHistogram()
        if UGH.SetRunNo( str(runNo) ):
            pass
        else:
            raise UserWarning, "runNo is invalid"
        UGH.SetDetParam("psd")
        
        Ei = float( self.minfo[0].GetValue() )
        minhw = float( self.minfo[1].GetValue() )
        maxhw = float( self.minfo[2].GetValue() )
        deltahw = float( self.minfo[3].GetValue() )
        if deltahw<=0.0 or minhw>maxhw or maxhw>=Ei :
            msg = "Energy Arguments are invalid."
            dial = wx.MessageDialog(None, msg, "Warning", wx.OK | wx.ICON_EXCLAMATION )
            dial.ShowModal()
            return
        
        minPhi = float( self.minfo[4].GetValue() )
        maxPhi = float( self.minfo[5].GetValue() )
        delPhi = float( self.minfo[6].GetValue() )
        if delPhi==0.0:
            delPhi=1.0;
            self.minfo[6].SetValue( str(delPhi) )
        
        hist_param = "hw,%f,%f,%f,%f"%(Ei,minhw,maxhw,deltahw)
        if UGH.SetConversionParameter( hist_param ):
            pass
        else:
            raise UserWarning, "hist param (%s) is invalid"%(hist_param)
        
        pfiles = UGH.MakeTempInfoFiles()
        if pfiles.size()<2:
            raise UserWarning, "fails to make tmp parameter files."
        
        btn_status = []
        for abtn in self.BtnCtrls:
            btn_status.append( abtn.IsEnabled() )
            abtn.Disable()
        self.frame.Update()
        
        lc = m.MakeDoubleVector(6)
        for i in range(6):
            lc[i] = float(self.sinfo[i].GetValue())

        uv = m.MakeDoubleVector(3)
        vv = m.MakeDoubleVector(3)
        for i in range(3):
            uv[i] = float(self.sinfo[6+i].GetValue())
            vv[i] = float(self.sinfo[9+i].GetValue())

        rotSteps = m.MakeDoubleVector()
        ## self.chRotationSteps is [self.chrotax1,self.chrotax2,self.chrotax3]
        for i in range(3):
            if self.sinfo[12+i].IsEnabled():
                axis_num = -1.0
                if self.chRotationSteps[i].GetCurrentSelection()==1:   # if Y-axis
                    axis_num = 1.0                             # 1 means Y-axis in VisualCalcSqe
                elif self.chRotationSteps[i].GetCurrentSelection()==2: # if Z-axis
                    axis_num = 2.0                             # 2 means Z-axis in VisualCalcSqe
                elif self.chRotationSteps[i].GetCurrentSelection()==3: # if X-axis
                    axis_num = 0.0                             # 0 means X-axis in VisualCalcSqe
                else:
                    break
                if axis_num > -1.0:
                    rotSteps.append( axis_num )                # axis number
                    rotSteps.append( float( self.sinfo[12+i].GetValue() ) )   # angle
            else:
                break

        va = m.MakeDoubleVector(16)
        titles = m.MakeStringVector(4)
        for id in range(4):
            for col in range(4):
                va[id*4 + col] = float( self.uv[id][col].GetValue() )
            print "id=",id,self.uv[id][4].GetValue()
            titles[id]= str( self.uv[id][4].GetValue() )

        hw_info = m.MakeDoubleVector(4)
        hw_info[0] = Ei
        hw_info[1] = deltahw
        hw_info[2] = minhw
        hw_info[3] = maxhw
        
        if minPhi>maxPhi:
            tmp_phi = minPhi
            minPhi = maxPhi
            maxPhi = tmp_phi
        if delPhi<0:
            delPhi = -1.0*delPhi
        
        phi_steps = m.MakeDoubleVector()
        phi = minPhi
        while( phi<maxPhi ):
            phi_steps.push_back( phi )
            phi += delPhi
        
        ranges = []
        for id in range(4):
            tmp = m.MakeDoubleVector(3)
            tmp[0] = float( self.D4MatRangeCtrl[3*id + 0].GetValue() )
            tmp[1] = float( self.D4MatRangeCtrl[3*id + 1].GetValue() )
            tmp[2] = float( self.D4MatRangeCtrl[3*id + 2].GetValue() )
            ranges.append( tmp )
            
        print "---------------------------------------"
        print "wfile, dfile=",pfiles[0], pfiles[1]
        print "lc = ",lc[0],lc[1],lc[2],lc[3],lc[4],lc[5]
        print "uv = ",uv[0],uv[1],uv[2]
        print "vv = ",vv[0],vv[1],vv[2]

        stime = time.time()        
        self.parent.D4Mat.D4Mat._CalcVirtualAngleInfo( pfiles[0], pfiles[1] )
        self.parent.D4Mat.D4Mat.AllocateVirtualMat(lc, uv, vv, rotSteps, va, hw_info, phi_steps,ranges[0],ranges[1],ranges[2],ranges[3],titles )
        print "  Allocation is finish : cost = ",time.time()-stime

        for i in range(4):
            vv = self.parent.D4Mat.D4Mat.PutAxRange(i)
            self.parent.D4MatRangeCtrl[ i*3 ].SetLabel(str(vv[0]))
            self.parent.D4MatRangeCtrl[ i*3 + 1 ].SetLabel(str(vv[1]))
            self.parent.D4MatRangeCtrl[ i*3 + 2 ].SetLabel(str(vv[2]))
            sv = self.parent.D4Mat.D4Mat.PutAxTitle(i)
            self.parent.D4MatTitle[ i ].SetValue( sv )
        
        self.parent.FileListBox.Clear()
        
        for btn_ctrl in self.parent.ButtonCtrlList:
            btn_ctrl.Enable(True)
        self.parent.isEnableSlice=True       #[inamura 170228]
        self.parent.CheckEnableSliceButton() #[inamura 170228]

        #for i in [4,5,6,9,10,11]:
        for i in [4,5,6,8,9,10]:
            self.parent.ButtonCtrlList[i].Enable(False)
        
        dlg = wx.MessageDialog(self.frame, "Making Virtual D4Matrix completed.","",wx.OK)
        ret = dlg.ShowModal()
        
        self.parent.D4MatrixFilePathCtrl.SetLabel("VIRTUAL D4Matrix Mode")
        self.parent.VirtualD4MatTextCtrl.SetLabel("VIRTUAL ( Ei=%g hw=[%g:%g:%g], Angle=[%g:%g:%g] )"%(Ei,minhw,maxhw,deltahw,minPhi,maxPhi,delPhi))
        self.parent.FileListBox.Clear()
        self.parent.FileListBox.Append( "Virtual Mode" )
        self.parent.OnAutoFill(None)
        
        for astatus,abtn in zip(btn_status,self.BtnCtrls):
            abtn.Enable(astatus)
        self.frame.Update()

        return
        
    #########################################
    def OnChoiceRotateAxes(self, evt):
        """
        CHOICE Event
        ##[inamura 150526]
        @param evt    Event
        @retval None
        """
        bt = evt.GetEventObject()
        if bt==self.chrotax1:
            if self.chrotax1.GetCurrentSelection()==0:
                self.sinfo[12].Enable(False)
                self.chrotax2.Enable(False)
                self.sinfo[13].Enable(False)
                self.chrotax3.Enable(False)
                self.sinfo[14].Enable(False)
            else:
                self.sinfo[12].Enable(True)
                self.chrotax2.Enable(True)
                if self.chrotax2.GetCurrentSelection()==0:
                    self.sinfo[13].Enable(False)
                    self.chrotax3.Enable(False)
                    self.sinfo[14].Enable(False)
                else:
                    self.sinfo[13].Enable(True)
                    self.chrotax3.Enable(True)
                    if self.chrotax3.GetCurrentSelection()==0:
                        self.sinfo[14].Enable(False)
                    else:
                        self.sinfo[14].Enable(True)
                        
        elif bt==self.chrotax2:
            if self.chrotax1.GetCurrentSelection()==0:
                self.sinfo[13].Enable(False)
                self.chrotax3.Enable(True)
                self.sinfo[14].Enable(False)
            else:
                if bt.GetCurrentSelection()==0:
                    self.sinfo[13].Enable(False)
                    self.chrotax3.Enable(False)
                    self.sinfo[14].Enable(False)
                else:
                    self.chrotax3.Enable(True)
                    self.sinfo[13].Enable(True)
                    if self.chrotax3.GetCurrentSelection()==0:
                        self.sinfo[14].Enable(False)
                    else:
                        self.sinfo[14].Enable(True)
                        
        else:
            if (self.chrotax1.GetCurrentSelection()==0) or (self.chrotax2.GetCurrentSelection()==0) or (bt.GetCurrentSelection()==0):
                self.sinfo[14].Enable(False)
            else:
                self.sinfo[14].Enable(True)
        
            

    ##########################################################
    def OnClear(self, evt):
        """
        """
        print self.CommentHead+"OnClear"

    #################################
    def OnMakeNewD4Mat(self,evt):
        """
        """
        # open directory dialog
        dlg = wx.FileDialog( self.frame, 'Choose File to save D4Matrix parameters ...', defaultDir=os.getcwd(), defaultFile='D4Mat.xml', wildcard='xml File(*.xml)|*.xml',style=wx.SAVE )

        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return

        # get path
        filepath = dlg.GetPath()
        dir = os.path.dirname(filepath)
        #print "filepath=",filepath
        #print "dir=",dir
        
        ax_list=[]
        title_list=[]
        for ax in range(4):
            ra = []
            for i in range(3):
                ra.append( float( self.D4MatRangeCtrl[ ax*3 + i ].GetValue() ) )
            ax_list.append(ra)
            title_list.append( self.D4MatTitle[ax].GetValue() )

        pre_label = self.parent.StatusTextCtrl.GetLabel()
        self.parent.StatusTextCtrl.SetLabel( "Making New D4Matrix files..." )
        self.parent.frame.Refresh()
        self.parent.frame.Update()
        self.frame.Refresh()
        self.frame.Update()
        
        btn_status = []
        for abtn in self.BtnCtrls:
            btn_status.append( abtn.IsEnabled() )
            abtn.Disable()
        self.frame.Update()

        self.parent.D4Mat.new( ax_list[0], ax_list[1], ax_list[2], ax_list[3], title_list, str(filepath) )
        ##self.parent.D4Mat.D4Mat._MakeNewParamXml( str(filepath)+".xml" ) ##[inamura 110411]
        for ax in ax_list:
            print self.CommentHead,ax
        
        self.parent.D4Mat.close()
        flag,msg = self.parent._OpenD4Matrix(str(filepath))
        if not flag:
            dial = wx.MessageDialog(None, msg, "Warning", wx.OK | wx.ICON_EXCLAMATION )
            dial.ShowModal()
            return

        self.parent.StatusTextCtrl.SetLabel( pre_label )
        
        for astatus,abtn in zip(btn_status,self.BtnCtrls):
            abtn.Enable(astatus)
        self.frame.Update()

        dlg = wx.MessageDialog(self.frame, "Making New D4Matrix files completed. Push OK to close this VirtualD4Mat Setting","",wx.OK)
        ret = dlg.ShowModal()
        self.parent.VirtualD4MatTextCtrl.SetLabel("No Info")
        self.OnClose(None)

        
    #################################
    def OnEstimateSize(self,evt):
        """
        """
        num_list=[]
        for ax in range(4):
            ra = []
            for i in range(3):
                ra.append( float( self.D4MatRangeCtrl[ ax*3 + i ].GetValue() ) )
            num = int( (ra[1]-ra[0])/ra[2] )
            num_list.append(num)
        
        size_bit = (num_list[0]+1)*(num_list[1]+1)*(num_list[2]+1)*(num_list[3]+1)*3.0*32.0
        size_MB = size_bit/8.0/1024.0/1024.0
        msg = "Estimated Size of D4Matrix data is %f MB"%size_MB
        dlg = wx.MessageDialog(self.frame, msg,"",wx.OK)
        ret = dlg.ShowModal()
        return

    ##########################################################
    def OnClose(self, evt):
        """
        Close button event
        @param  evt event information
        @retval None
        """        
        self.parent.StatusTextCtrl.SetLabel( "Done." )
        self.frame.Destroy()



#<--[inamrua 160621]

##<-[inamura 160523]        
###################################
## Main Window to treat D4Mat
###################################
class D4MatSlicer(wx.Frame):
    __version__ = 'v110411'
    def __init__(self, parent):
        """
        Constructor
        @param parent parent window id
        
        """
        
        ## Make window and set Event handler
        self.frame = get_resources().LoadFrame(parent,"D4MatGUIMainFrame")

        ## bind Button
        self.frame.Bind( wx.EVT_BUTTON, self.OnOpenD4Matrix, id=xrc.XRCID("btOpenD4Matrix"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnCloseD4Matrix, id=xrc.XRCID("btCloseMatrix"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnMakeVirtualMat, id=xrc.XRCID("btMakeVirtualMat")) #[inamura 160621]
        self.frame.Bind( wx.EVT_BUTTON, self.OnClearColumn, id=xrc.XRCID("btClearColumn"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnSaveTitles, id=xrc.XRCID("btSaveTitles"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnCloseD4Matrix, id=xrc.XRCID("btCloseMatrix"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnAddSingleFile, id=xrc.XRCID("btAddSingleFile"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnAddFilesInDir, id=xrc.XRCID("btAddFilesInDir"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnRemoveFile, id=xrc.XRCID("btRemoveFile"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnAutoFill, id=xrc.XRCID("btAutoFill"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnAutoFill, id=xrc.XRCID("btAutoFillAx1") ) #[inamura 160616]
        self.frame.Bind( wx.EVT_BUTTON, self.OnAutoFill, id=xrc.XRCID("btAutoFillAx2") ) #[inamura 160616]
        self.frame.Bind( wx.EVT_BUTTON, self.OnAutoFill, id=xrc.XRCID("btAutoFillAx3") ) #[inamura 160616]
        self.frame.Bind( wx.EVT_BUTTON, self.OnAutoFill, id=xrc.XRCID("btAutoFillAx4") ) #[inamura 160616]
        self.frame.Bind( wx.EVT_BUTTON, self.OnSlice, id=xrc.XRCID("btSlice") )
        self.frame.Bind( wx.EVT_BUTTON, self.OnSlice3d, id=xrc.XRCID("btSlice3D") ) #[inamura 160616]
        self.frame.Bind( wx.EVT_BUTTON, self.OnOut3DText, id=xrc.XRCID("btOut3DText") )
        self.frame.Bind( wx.EVT_BUTTON, self.OnDumpToText, id=xrc.XRCID("btDumpToText") )
        self.frame.Bind( wx.EVT_BUTTON, self.OnDumpToBin, id=xrc.XRCID("btDumpToBin") )

        self.D4MatrixFilePathCtrl = xrc.XRCCTRL( self.frame, "stPathToD4MatrixFiles" )
        self.StatusTextCtrl = xrc.XRCCTRL( self.frame, "txtStatus" )
        self.VirtualD4MatTextCtrl = xrc.XRCCTRL( self.frame, "txtvmsg" ) #[inamura 160627]
        
        self.ButtonCtrlList = []
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btOpenD4Matrix" ) ) # 0
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btClearColumn" ) )  # 1
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btSaveTitles" ) )   # 2
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btCloseMatrix" ) )  # 3
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btAddSingleFile" ) )# 4
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btAddFilesInDir" ) )# 5
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btRemoveFile" ) )   # 6
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btAutoFill" ) )     # 7
        #self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btSlice" ) )        # 8
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btOut3DText" ) )    # 9->8
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btDumpToText" ) )   # 10->9
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btDumpToBin" ) )    # 11->10
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btAutoFillAx1" ) ) ##[inamura 160616]
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btAutoFillAx2" ) ) ##[inamura 160616]
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btAutoFillAx3" ) ) ##[inamura 160616]
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btAutoFillAx4" ) ) ##[inamura 160616]
        ##[inamura 170227]-->
        #self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btSlice3D" ) )
        self.SliceButtonCtrlList=[xrc.XRCCTRL( self.frame, "btSlice" ), xrc.XRCCTRL( self.frame, "btSlice3D" ) ]
        self.isEnableSlice = False
        ##<--[inamura 170227]
        for btn_ctrl in self.ButtonCtrlList:
            btn_ctrl.Enable(False)
        self.SliceButtonCtrlList[0].Enable(False) #[inamura 170227]
        self.SliceButtonCtrlList[1].Enable(False) #[inamura 170227]
        for i in range(2):
            self.ButtonCtrlList[i].Enable()
        
        mpanel = xrc.XRCCTRL( self.frame, "D4MatSetPanel" )
        self.D4MatRangeCtrl = []
        self.D4MatTitle = []
        for i in range(12):
            self.D4MatRangeCtrl.append( xrc.XRCCTRL( mpanel, ( "txt%d" % (i+1) ) ) )
            
        for i in range(4):
            self.D4MatTitle.append( xrc.XRCCTRL( mpanel, ( "v%dtitle" % (i+1) ) ) )
            

        spanel = xrc.XRCCTRL( self.frame, "SlicePanel" )
        self.SliceAxis = []
        self.SliceRange = []
        self.SliceFoldCB = []
        self.SliceFoldDiag = []

        self.cbdiagfold = xrc.XRCCTRL(spanel, 'cbDiagFoldType')
        self.cbdiagfold.Bind(wx.EVT_CHOICE, self.OnComboDiagFold, self.cbdiagfold)
        for i in range(4):
            self.SliceAxis.append( xrc.XRCCTRL( spanel, ( "v%dcb" % (i+1) ) ) )
            #self.SliceAxis[-1].Bind(wx.EVT_COMBOBOX, self.OnSliceAxis, self.SliceAxis[-1])
            self.SliceFoldCB.append( xrc.XRCCTRL( spanel, ( "txtFold%d" % (i+1) ) ) )     ##[inamura 160616]
            self.SliceFoldDiag.append( xrc.XRCCTRL( spanel, ( "ckbUseAx%1dFold" % (i+1) ) ) )
        for i in range(12):
            self.SliceRange.append( xrc.XRCCTRL( spanel, ( "txt%d" % (i+1) ) ) )

        ## Axis type bind #[inamura 170228]-->
        for i in range(4):
            self.SliceAxis[i].Bind(wx.EVT_CHOICE, self.OnChangeSliceAxis, self.SliceAxis[i])
        #<--[inamura 170228]
        
        menubar = get_resources().LoadMenuBar("MainMenuBar")
        self.frame.SetMenuBar( menubar )
        self.frame.Bind(wx.EVT_MENU, self.OnOpenMakeNewFrame, id=xrc.XRCID('menu_new_matrix'))
        self.frame.Bind(wx.EVT_MENU, self.OnOpenD4Matrix, id=xrc.XRCID('menu_open_matrix'))
        self.frame.Bind(wx.EVT_MENU, self.OnOpenRebinFrame, id=xrc.XRCID('menu_rebin_matrix'))
        #self.frame.Bind(wx.EVT_MENU, self.OnSaveDataAsText, id=xrc.XRCID('menu_saveas_text'))
        self.frame.Bind(wx.EVT_MENU, self.OnClose, id=xrc.XRCID('menu_exit'))
        
        self.FileListBox = xrc.XRCCTRL( self.frame, "lbFileList" )
        
        ## initialize
        for i in range(4):
            """
            self.D4MatRangeCtrl[ i*3 ].SetValue("-2.0")
            self.D4MatRangeCtrl[ i*3 + 1 ].SetValue("2.0")
            self.D4MatRangeCtrl[ i*3 + 2 ].SetValue("0.05")
            """
            self.D4MatRangeCtrl[ i*3 ].SetLabel("-2.0")
            self.D4MatRangeCtrl[ i*3 + 1 ].SetLabel("2.0")
            self.D4MatRangeCtrl[ i*3 + 2 ].SetLabel("0.05")
            if i != 3:
                self.D4MatTitle[ i ].SetValue( ("Q%d" % (i+1) ) )
            else:
                self.D4MatTitle[ i ].SetValue( "hw" )
                
                            
        # Justify the alignment of controls on OSX
        if sys.platform == "darwin":
            self.frame.SetSizeWH(540,740)
        if sys.platform =="win32":
            self.frame.SetSizeWH(540,720)

        self.frame.Show()
        
        self.D4Mat = D4Mat.D4Mat()
        self.ParamFilePath = ""
        self.CommentHead = "D4MatGUI >>> "
        self.M2Plot = None
        self.MakeVirtualFrame = None
        self.SliceViewer3d = None #[inamura 170227]

    #################################        
    def OnOpenD4Matrix(self,evt):
        """
        """
        print self.CommentHead,"OnOpenD4Matrix"
        ##[inamura 160621]-->
        # isVirtual? 
        if self.D4Mat.D4Mat.isVirtualMode():
            dlg = wx.MessageDialog(self.frame, "Now You use Virtual Matrix. Are you sure to delete it ?", 'Question', wx.YES_NO|wx.NO_DEFAULT|wx.ICON_QUESTION)
            if dlg.ShowModal() !=wx.ID_YES:
                return
        try:
            if self.MakeVirtualFrame != None:
                self.MakeVirtualFrame.OnClose(None)
        except:
            pass
        self.VirtualD4MatTextCtrl.SetLabel("No Info")
        ##<--[inamura 160621]
        # open dialog
        dlg = wx.FileDialog(self.frame, 'Open D4Matrix param file ...', os.getcwd(), style=wx.OPEN, wildcard='XML(*.xml)|*.xml')

        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return

        # get file path
        self.ParamFilePath = str( dlg.GetPath() )
        #dir = os.path.dirname(filepath)
        flag,msg = self._OpenD4Matrix()
        if not flag:
            dial = wx.MessageDialog(None, msg, "Warning", wx.OK | wx.ICON_EXCLAMATION )
            dial.ShowModal()
            return


    ################################
    def _OpenD4Matrix(self,filepath=""):
        if filepath=="":
            if self.ParamFilePath == "":
                msg = "Not set param file path.."
                print self.CommentHead + msg
                return (False,msg)
        else:
            if os.path.exists( filepath ):
                self.ParamFilePath = filepath
            else:
                msg = "Not found such file="+filepath
                print self.CommentHead + msg
                return (False,msg)
        
        ret = self.D4Mat.open(self.ParamFilePath)
        if (ret!=-1):
            self.D4MatrixFilePathCtrl.SetLabel(self.ParamFilePath)
        else:
            msg = " File is invalid. ("+self.ParamFilePath+")"
            print self.CommentHead + msg
            return (False,msg)

        for i in range(4):
            vv = self.D4Mat.D4Mat.PutAxRange(i)
            """
            self.D4MatRangeCtrl[ i*3 ].SetValue(str(vv[0]))
            self.D4MatRangeCtrl[ i*3 + 1 ].SetValue(str(vv[1]))
            self.D4MatRangeCtrl[ i*3 + 2 ].SetValue(str(vv[2]))
            """
            self.D4MatRangeCtrl[ i*3 ].SetLabel(str(vv[0]))
            self.D4MatRangeCtrl[ i*3 + 1 ].SetLabel(str(vv[1]))
            self.D4MatRangeCtrl[ i*3 + 2 ].SetLabel(str(vv[2]))
            sv = self.D4Mat.D4Mat.PutAxTitle(i)
            self.D4MatTitle[ i ].SetValue( sv )

        self.FileListBox.Clear()
        FileCompsVec = self.D4Mat.D4Mat.PutFileComponents()
        for i in range(FileCompsVec.size()):
            self.FileListBox.Append( FileCompsVec[i] )
        
        for btn_ctrl in self.ButtonCtrlList:
            btn_ctrl.Enable(True)
        self.isEnableSlice=True       #[inamura 170228]
        self.CheckEnableSliceButton() #[inamura 170228]
        
        self.OnAutoFill()
        return (True,"")

    #################################
    def OnCloseD4Matrix(self,evt):
        """
        """
        print self.CommentHead+"OnCloseMatrix"
        
        self._CloseD4Matrix()

        
    #[inamrua 160621]-->
    #################################
    def OnMakeVirtualMat(self,evt):
        """
        """
        print self.CommentHead+"OnMakeVirtualMat"
        if self.D4Mat.D4Mat.isMatOpened():
            dlg = wx.MessageDialog(self.frame, "Now You open D4Matrix. Are you sure to close it ?", 'Question', wx.YES_NO|wx.NO_DEFAULT|wx.ICON_QUESTION)
            if dlg.ShowModal() !=wx.ID_YES:
                return
        try:
            self.OnCloseD4Matrix(None)
        except:
            pass
            
        self.StatusTextCtrl.SetLabel( "Opening window for Making Virtual or New D4Matrix." )
        self.MakeVirtualFrame = MakeVirtualD4MatrixFrame(self)
        
    #<--[inamura 160621]
    #################################
    def OnClearColumn(self,evt):
        """
        """
        print self.CommentHead+"OnClearColumn"
        
        for i in range(4):
            vv = self.D4Mat.D4Mat.PutAxRange(i)
            """
            self.D4MatRangeCtrl[ i*3 ].SetValue(str(vv[0]))
            self.D4MatRangeCtrl[ i*3 + 1 ].SetValue(str(vv[1]))
            self.D4MatRangeCtrl[ i*3 + 2 ].SetValue(str(vv[2]))
            """
            self.D4MatRangeCtrl[ i*3 ].SetLabel(str(vv[0]))
            self.D4MatRangeCtrl[ i*3 + 1 ].SetLabel(str(vv[1]))
            self.D4MatRangeCtrl[ i*3 + 2 ].SetLabel(str(vv[2]))
            sv = self.D4Mat.D4Mat.PutAxTitle(i)
            self.D4MatTitle[ i ].SetValue( sv )

    #################################
    def OnSaveTitles(self,evt):
        """
        """
        titles = []
        for i in range(4):
            tt = self.D4MatTitle[ i ].GetValue()
            titles.append( tt )
        self.D4Mat.setTitles( titles, True )

    #################################
    def OnAddSingleFile(self,evt):
        """
        """
        print self.CommentHead+"OnAddSingleFile"

        # open dialog
        dlg = wx.FileDialog(self.frame, 'Choose VisualCont2 output file ...', os.getcwd(),
                            style=wx.OPEN, wildcard='Binary(*.vbin)|*.vbin|TEXT(*.txt)|*.txt')

        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return
        
        filepath = str( dlg.GetPath() )
        self.StatusTextCtrl.SetLabel( "Loading file %s" % (filepath ) )
        for btn_ctrl in self.ButtonCtrlList:
            btn_ctrl.Enable(False)
        self.isEnableSlice=False       #[inamura 170228]
        self.CheckEnableSliceButton() #[inamura 170228]

        self.frame.Refresh()
        self.frame.Update()
        
        self.D4Mat.addFromFile( filepath )
        self.StatusTextCtrl.SetLabel( "Done." )
        
        self.FileListBox.Append( filepath )

        for btn_ctrl in self.ButtonCtrlList:
            btn_ctrl.Enable(True)
        self.isEnableSlice=True       #[inamura 170228]
        self.CheckEnableSliceButton() #[inamura 170228]
        

    #################################
    def OnAddFilesInDir(self,evt):
        """
        """
        print self.CommentHead+"OnAddFilesInDir"
        
        #dlg = wx.DirDialog(self.frame, "Select Directory including VisualCont2 output files ...",os.getcwd(), style=wx.OPEN)
        dlg = wx.DirDialog(self.frame, "Select Directory including VisualCont2 output files ...",os.getcwd())
        if dlg.ShowModal() !=wx.ID_OK:
            return
        
        print self.CommentHead,"DIR=",dlg.GetPath()

        #check selected dir
        filedir = str(dlg.GetPath())
        if not os.path.exists( filedir ):
            print self.CommnetHead + "Cannot find such directory, "+filedir
            raise
        
        #show progress bar
        self.D4Mat.progress=0 #[inamura 180412]
        pdg = wx.ProgressDialog("Loading","Loading VisualCont2 output text files :",len( os.listdir(filedir) ))
        #start sub-thread
        #th = threading.Thread( target=self.D4Mat.addFromTextDir,args=(str( dlg.GetPath() ), ) )
        th = threading.Thread( target=self.D4Mat.addFromDirectry,args=(str( dlg.GetPath() ), ) )
        #th.setDaemon(True)
        th.start()
        # progress
        pdg.Update(self.D4Mat.progress)
        while(True):                           ##[inamura 170116]-->
            wx.Sleep(0.5)
            if (self.D4Mat.progress > self.D4Mat.progress_max):
                break                          ##<--[inamura 170116]
            pdg.Update(self.D4Mat.progress)
        pdg.Destroy()
        
        #self.D4Mat.addFromTextDir( str( dlg.GetPath() ) )

        """
        self.FileListBox.Clear()
        
        FileCompsVec = self.D4Mat.D4Mat.PutFileComponents()
        for i in range(FileCompsVec.size()):
            self.FileListBox.Append( FileCompsVec[i] )
        """
        for file in self.D4Mat.AddedFileList:
            self.FileListBox.Append( file )
        
        ##[inamura 170116]-->
        msg = "Finished: Load vbin files in "+filedir
        dial = wx.MessageDialog(None, msg, "Message", wx.OK | wx.ICON_INFORMATION )
        dial.ShowModal()
        return
        ##<--[inamura 170116]
        
    #################################
    def OnRemoveFile(self,evt):
        """
        """
        print self.CommentHead+"OnRemoveFile"
        selected_list = self.FileListBox.GetSelections()
        
        if len(selected_list)==0:
            return
        else:
            print self.CommentHead,"selected ",selected_list
            print self.CommentHead,"len=",len(selected_list)
            last_index = len(selected_list)-1
            for i in range(last_index+1):
                ind = selected_list[ last_index - i ]
                filepath = self.FileListBox.GetString(ind)
                print self.CommentHead,"Removing File = ",filepath
                ret = self.D4Mat.subtractText( filepath )
                if ret==0:
                    self.FileListBox.Delete(ind)
                

    #################################
    def OnAutoFill(self,evt=None):
        """
        """
        print self.CommentHead+"OnAutoFill"
        #[inamura 160621]-->
        if (evt==None):
            bt_name = "btAutoFill"
        else:
            bt_name = evt.GetEventObject().GetName()
        bt_list = None
        if bt_name=="btAutoFill":
            bt_list=[0,1,2,3]
        elif bt_name=="btAutoFillAx1":
            bt_list=[0]
        elif bt_name=="btAutoFillAx2":
            bt_list=[1]
        elif bt_name=="btAutoFillAx3":
            bt_list=[2]
        elif bt_name=="btAutoFillAx4":
            bt_list=[3]
        else:
            return
        #<--[inamura 160621]
        #for i in range(12):
        #    self.SliceRange[i].SetValue( self.D4MatRangeCtrl[i].GetValue() )
        #for i in range(4): #[inamura 160616]
        for i in bt_list:
            vv = self.D4Mat.D4Mat.PutAxRange(i)
            for j in range(3):
                self.SliceRange[ 3*i + j ].SetValue( str(vv[j]) )

    #[inamura 170228]-->
    #################################
    def OnChangeSliceAxis(self,evt):
        """
        """
        self.CheckEnableSliceButton()

    #########################################
    def CheckEnableSliceButton(self):
        """
        """
        if self.isEnableSlice:
            flags = [0,0,0,0]
            for i in range(4):
                curNo = self.SliceAxis[i].GetCurrentSelection()
                flags[curNo] +=1
            if flags==[1,1,0,2]:
                self.SliceButtonCtrlList[0].Enable(True)
                self.SliceButtonCtrlList[1].Enable(False)
            elif flags==[1,1,1,1]:
                self.SliceButtonCtrlList[0].Enable(False)
                self.SliceButtonCtrlList[1].Enable(True)
            else:
                self.SliceButtonCtrlList[0].Enable(False)
                self.SliceButtonCtrlList[1].Enable(False)
        else:
            self.SliceButtonCtrlList[0].Enable(False)
            self.SliceButtonCtrlList[1].Enable(False)
            
        if self.D4Mat.D4Mat.isVirtualMode():
            self.SliceButtonCtrlList[1].Enable(False)
    #<--[inamura 170228]
    #########################################
    def OnComboDiagFold(self, evt):
        """
        """
        index = 0
        if isinstance( evt, int ):
            index = evt
        else:
            bt = evt.GetEventObject()
            index = bt.GetSelection()
        
        if index==0:
            for obj in self.SliceFoldDiag:
                obj.SetValue(False)
                obj.Enable(False)
        else:
            for obj in self.SliceFoldDiag:
                obj.Enable(True)
    
    #################################
    def OnSlice(self,evt):
        """
        """
        print self.CommentHead+"OnSlice"
        #[inamura 160630]--> #check X,Y and Z
        flags=[0,0,0,0]
        for i in range(4):
            ax = self.SliceAxis[i].GetCurrentSelection()
            if ax == 0: # X
                flags[0]+=1
            elif ax == 1: # Y
                flags[1]+=1
            elif ax == 2: # Z
                flags[2]+=1
            elif ax == 3: # T
                flags[3]+=1
            else:
                raise
        if flags!=[1,1,0,2]:
            msg = "Axis type selections are invalid."
            dial = wx.MessageDialog(None, msg, "Warning", wx.OK | wx.ICON_EXCLAMATION )
            dial.ShowModal()
            return
        #<--[inamrua 160630]
        
        Axr = []
        type_axes = []
        foldings = []
        for i in range(4):
            #ax = self.SliceAxis[i].GetSelection() #[inamura 160616]
            ax = self.SliceAxis[i].GetCurrentSelection() 
            if ax == 0:
                type_axes.append("X")
            elif ax == 1:
                type_axes.append("Y")
            elif ax == 2:
                type_axes.append("Z") #[inamura 160616]
            elif ax == 3:
                type_axes.append("T")
            else:
                raise
            
            #[inamura 160616]-->
            #if self.SliceFoldCB[i].GetValue():
            #    foldings.append(1)
            #else:
            #    foldings.append(0)
            foldings.append( float( self.SliceFoldCB[i].GetValue() ) )
            #<--[inamura 160616]
            
            ar = []
            ar.append( float( self.SliceRange[i*3].GetValue() ) )
            ar.append( float( self.SliceRange[i*3+1].GetValue() ) )
            Axr.append(ar)
        
        DiagCond = self.cbdiagfold.GetSelection()
        print "### DiagCond = ",DiagCond
        if DiagCond==wx.NOT_FOUND:
            print "Not Secected"
        elif (DiagCond==1) or (DiagCond==2) or (DiagCond==3):
            foldings.append( float( DiagCond ) )
            ax_fold = []
            for i in range(len(self.SliceFoldDiag)):
                if self.SliceFoldDiag[i].GetValue():
                    ax_fold.append(i)
            if len(ax_fold)!=2:
                print "the number of Checked must be 2"
                return
            else:
                foldings.append( float( ax_fold[0] ) )
                foldings.append( float( ax_fold[1] ) )
        else:
            foldings.append(-1.0)
            foldings.append(0.0)
            foldings.append(0.0)

        self.D4Mat.sliceMat(Axr[0],Axr[1],Axr[2],Axr[3], type_axes, foldings )

        d2 = M2P.D2Matrix()
        map = d2.ReadMatrix( self.D4Mat.SlicedECM )

        titles = ["","","","",""]
        titles[0] = "D4Matrix Slice"
        
        for i,c in enumerate(type_axes):
            title = self.D4MatTitle[i].GetValue()
            if c=="X":
                titles[2] = title
            elif c=="Y":
                titles[3] = title
            else:
                titles[1] += "Ax%d : %s\n" % (i,title)
                titles[1] += "     from %s to %s\n" % (self.SliceRange[i*3].GetValue(),self.SliceRange[i*3+1].GetValue())
        tmp_str = "Fold:"
        cnt_fld = 0
        for j in range(4):
            if float(self.SliceFoldCB[j].GetValue())!=-1.0:
                if cnt_fld>1:
                    tmp_str += "\n      Ax%d[%g]"%(j+1,float(self.SliceFoldCB[j].GetValue()))
                else:
                    tmp_str += " Ax%d[%g]"%(j+1,float(self.SliceFoldCB[j].GetValue()))
                cnt_fld+=1

        if tmp_str=="Fold:":
            pass
        else:
            titles[1] += tmp_str
        
        titles[4] = "Intensity"
        
        if self.M2Plot == None:
            self.M2Plot = Plot2DMap(0, "D4MatSlicer", self.frame)
            
        self.M2Plot.PlotMap( titles, map, type_axes)

    ##[inamura 170227]-->
    #################################
    def OnSlice3d(self,evt):
        """
        """
        #print self.CommentHead+"OnSlice3d"
        flags=[0,0,0,0]
        for i in range(4):
            ax = self.SliceAxis[i].GetCurrentSelection()
            if ax == 0: # X
                flags[0]+=1
            elif ax == 1: # Y
                flags[1]+=1
            elif ax == 2: # Z
                flags[2]+=1
            elif ax == 3: # T
                flags[3]+=1
            else:
                raise
        if flags!=[1,1,1,1]:
            msg = "Axis type selections are invalid."
            dial = wx.MessageDialog(None, msg, "Warning", wx.OK | wx.ICON_EXCLAMATION )
            dial.ShowModal()
            return
        
        Axr = []
        type_axes = []
        foldings = []
        name_axes = ["-","-","-"]
        for i in range(4):
            ax = self.SliceAxis[i].GetCurrentSelection()
            if ax == 0:
                type_axes.append("X")
                name_axes[0] = self.D4MatTitle[i].GetValue()
            elif ax == 1:
                type_axes.append("Y")
                name_axes[1] = self.D4MatTitle[i].GetValue()
            elif ax == 2:
                type_axes.append("Z")
                name_axes[2] = self.D4MatTitle[i].GetValue()
            elif ax == 3:
                type_axes.append("T")
            else:
                raise
            
            foldings.append( float( self.SliceFoldCB[i].GetValue() ) )
            
            ar = []
            ar.append( float( self.SliceRange[i*3].GetValue() ) )
            ar.append( float( self.SliceRange[i*3+1].GetValue() ) )
            Axr.append(ar)

        keys_axes = ["Vx","Vy","Vz"]
        print "Keys=",keys_axes[0],keys_axes[1],keys_axes[2]
        self.D4Mat.sliceMat3d(Axr[0],Axr[1],Axr[2],Axr[3], type_axes, foldings, keys_axes )
        
        import vis.SliceViewer3D as SV
        if self.SliceViewer3d==None:
            self.SliceViewer3d = SV.SliceViewer()
        
        if self.SliceViewer3d.setData( 0, self.D4Mat.SlicedECM, "Sliced data in D4MatSlicer" ):
            pass
        else:
            del self.SliceViewer3d
            self.SliceViewer3d = SV.SliceViewer()
            self.SliceViewer3d.setData( 0, self.D4Mat.SlicedECM, "Sliced data in D4MatSlicer" )

        titles = []
        for a_name in name_axes:
            titles.append( a_name.replace(" ","") )
        self.SliceViewer3d.setAxesTitles( titles[0], titles[1], titles[2] )
        
    ##<--[inamura 170227]
        
    #################################
    def OnOut3DText(self,evt):
        """
        """
        print self.CommentHead+"OnOut3DText"
        #[inamura 160630]--> #check X,Y and Z
        flags=[0,0,0,0]
        for i in range(4):
            ax = self.SliceAxis[i].GetCurrentSelection()
            if ax == 0: # X
                flags[0]+=1
            elif ax == 1: # Y
                flags[1]+=1
            elif ax == 2: # Z
                flags[2]+=1
            elif ax == 3: # T
                flags[3]+=1
            else:
                raise
        if flags!=[1,1,1,1]:
            msg = "Invalid axis types. You must choose axes X,Y,Z and T for each."
            dial = wx.MessageDialog(None, msg, "Warning", wx.OK | wx.ICON_EXCLAMATION )
            dial.ShowModal()
            return
        #<--[inamrua 160630]
        
        self.MaskValInfo=[True,False,0.0] # [<Ignore Points with MaskVal>, <Use Given MasVal>, <MaskVal>]
        dlg = SetSaveAsTextDialog( self, self.frame )
        
        """
        maskValStr=""        
        if (self.MaskValInfo[1]):
            maskValStr = str( self.MaskValInfo[2] )
        else:
            maskValStr = "MASKVALUE"

        # open dialog
        dlg = wx.FileDialog(self.frame, 'Choose 3D text output file ...', os.getcwd(),
                            style=wx.SAVE, wildcard='TEXT(*.txt)|*.txt')
        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return
        
        #filepath = str( dlg.GetPath() )
        filepath = str( dlg.GetFilename() )
        self.StatusTextCtrl.SetLabel( "Out3D saves file %s" % (filepath ) )
        self.frame.Refresh()
        self.frame.Update()
        
        Axr = []
        type_axes = []
        foldings = []
        for i in range(4):
            ax = self.SliceAxis[i].GetSelection()
            if ax == 0:
                type_axes.append("X")
            elif ax == 1:
                type_axes.append("Y")
            elif ax == 2:
                type_axes.append("T")
            elif ax == 3:
                type_axes.append("Z")
            else:
                raise
            
            if self.SliceFoldCB[i].GetValue():
                foldings.append(1)
            else:
                foldings.append(0)
            
            ar = []
            ar.append( float( self.SliceRange[i*3].GetValue() ) )
            ar.append( float( self.SliceRange[i*3+1].GetValue() ) )
            Axr.append(ar)
            
        import Manyo as mm
        a1v = mm.ListToDoubleVector( Axr[0] )
        a2v = mm.ListToDoubleVector( Axr[1] )
        a3v = mm.ListToDoubleVector( Axr[2] )
        a4v = mm.ListToDoubleVector( Axr[3] )
        def_axes = mm.MakeStringVector()
        fld = mm.ListToInt4Vector( foldings )
        for Axtype in type_axes:
            def_axes.append(Axtype)

        #self.D4Mat.D4Mat.OutputText3D(a1v, a2v, a3v, a4v, def_axes, fld, str(filepath) )
        self.D4Mat.D4Mat.OutputText3D(a1v, a2v, a3v, a4v, def_axes, fld, str(filepath), self.MaskValInfo[0], maskValStr )
        self.StatusTextCtrl.SetLabel( "Ready" )
        """

    #################################
    def OnDumpToText(self,env):
        """
        """
        self.DumpToFile(True)
    #################################
    def OnDumpToBin(self,env):
        """
        """
        self.DumpToFile(False)
    #################################
    def DumpToFile(self,isText):
        """
        """
        print self.CommentHead+"OnSlice::DumpToFile"
        
        Axr = []
        type_axes = []
        foldings = []
        flag_axes = [0,0,0,0]
        for i in range(4):
            #ax = self.SliceAxis[i].GetSelection() #[inamura 160616]
            ax = self.SliceAxis[i].GetCurrentSelection()
            if ax == 0:
                type_axes.append("X")
                flag_axes[0] +=1
            elif ax == 1:
                type_axes.append("Y")
                flag_axes[1] +=1
            elif ax == 2:
                type_axes.append("Z") #[inamura 160616]
                flag_axes[2] +=1
            elif ax == 3:
                type_axes.append("T") #[inamura 160616]
                flag_axes[3] +=1
            else:
                raise
            
            ##[inamura 160616]-->
            #if self.SliceFoldCB[i].GetValue():
            #    foldings.append(1)
            #else:
            #    foldings.append(0)
            foldings.append( float( self.SliceFoldCB[i].GetValue() ) )
            #<--[inamura 160616]
            
            ar = []
            ar.append( float( self.SliceRange[i*3].GetValue() ) )
            ar.append( float( self.SliceRange[i*3+1].GetValue() ) )
            Axr.append(ar)
            
        if flag_axes!=[1,1,1,1]:
            msg = "Invalid axes types. You must choose X,Y,Z and T for each."
            dial = wx.MessageDialog(None, msg, "Warning", wx.OK | wx.ICON_EXCLAMATION )
            dial.ShowModal()
            return

        import Manyo as mm
        a1v = mm.ListToDoubleVector( Axr[0] )
        a2v = mm.ListToDoubleVector( Axr[1] )
        a3v = mm.ListToDoubleVector( Axr[2] )
        a4v = mm.ListToDoubleVector( Axr[3] )
        def_axes = mm.MakeStringVector()
        #fld = mm.ListToInt4Vector( foldings ) #[inamura 160616]
        fld = mm.ListToDoubleVector( foldings ) #[inamura 160616]
        for Axtype in type_axes:
            def_axes.append(Axtype)
        
        # open dialog
        dlg = None
        if (isText):
            dlg = wx.FileDialog(self.frame, 'Choose Dump Text output file ...', os.getcwd(),
                                style=wx.SAVE, wildcard='TEXT(*.txt)|*.txt')
        else:
            dlg = wx.FileDialog(self.frame, 'Choose Dump Binary output file ...', os.getcwd(),
                                style=wx.SAVE, wildcard='BIN(*.bin)|*.bin')
        
        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return
        
        btn_status = []
        for abtn in self.ButtonCtrlList:
            btn_status.append( abtn.IsEnabled() )
            abtn.Disable()
        self.SliceButtonCtrlList[0].Disable() ##[inamura 170228]
        self.SliceButtonCtrlList[1].Disable() ##[inamura 170228]

        #filepath = str( dlg.GetPath() )
        filepath = str( dlg.GetFilename() )
        self.StatusTextCtrl.SetLabel( "Dumping To save file %s" % (filepath ) )
        self.frame.Refresh()
        self.frame.Update()
        
        self.D4Mat.D4Mat.DumpD4MatToFile(a1v, a2v, a3v, a4v, def_axes, fld, str(filepath), isText )
        self.StatusTextCtrl.SetLabel( "Ready" )
        
        for astatus,abtn in zip(btn_status,self.ButtonCtrlList):
            abtn.Enable(astatus)
        self.CheckEnableSliceButton() #[inamura 170228]
        
        self.frame.Update()
        msg = "Finished Save 4D data to file file : "+filepath
        dial = wx.MessageDialog(None, msg, "Message", wx.OK | wx.ICON_INFORMATION )
        dial.ShowModal()


        
    #################################
    def OnOpenMakeNewFrame(self,evt):
        """
        """
        print self.CommentHead+"OnOpenMakeNewFrame"
        self.StatusTextCtrl.SetLabel( "Opening window for Making New D4Matrix." )
        dlg = MakeNewMatrixFrame(self)
        #self.StatusTextCtrl.SetLabel( "Done." )

        
    #################################
    def OnOpenRebinFrame(self,evt):
        """
        """
        print self.CommentHead+"OnOpenRebinFrame"
        self.StatusTextCtrl.SetLabel( "Opening window for Rebin D4Matrix." )
        dlg = RebinMatrixFrame(self)
        #self.StatusTextCtrl.SetLabel( "Done." )
        

    #################################
    def OnSaveDataAsText(self,evt):
        """
        """
        print self.CommentHead+"OnSaveDataAsText"


    #################################
    def OnClose(self,evt):
        """
        """
        print self.CommentHead+"OnClose"
        self._CloseD4Matrix()
        self.frame.Destroy()


    ################################
    def _CloseD4Matrix(self):
        """
        """
        self.D4Mat.close()
        
        self.ParamFilePath = ""
        self.D4MatrixFilePathCtrl.SetLabel("Not selected.")
        for i in range(2,len(self.ButtonCtrlList)):
            self.ButtonCtrlList[i].Enable(False)
        self.isEnableSlice=False       #[inamura 170228]
        self.CheckEnableSliceButton() #[inamura 170228]
        self.FileListBox.Clear()



#######################################
#  Plot2DMap
####################################### 
class Plot2DMap(object):
    """
    2次元マップ表示クラス
    """ 
    #########################################################
    def __init__(self, num, code, frame):
        """
        コンストラクタ
        @param  num  タブ No.
        @param  code  マップ画面タイトルに付する記号
        @param  frame  親フレーム
        @retval 無し
        """ 
        self.frame = frame
        self.code = code
        self.d2 = None
        self.type_axes=None
        if num == 0:
            self.ifi = UtilPlot.IFEvtProp(0)
            self.pno = 0  
        else:
            self.pno = (num+1) * 10  
            self.ifi = UtilPlot.IFEvtProp(self.pno) 

    #########################################
    def PlotMap(self, labels, map, type_axes):
        """
        2D マップ表示
        @param panel タブのインスタンス
        @param map　2D データ
        @retval 無し
        """
        # D2 プロッタが表示されているか
        if self.d2 == None:
            # 2Dプロッタフレームクローズイベントのリスナー登録
            self.ifi.AddListner('2dclosed',  self.OnNotify2DClose)
            self.d2 = D2Vis.MapD2Frame(self.frame , map, self.pno, self.code) 
            self.type_axes=type_axes[:] ##[inamura 160823]
        else:
            # データ変更
            if self.type_axes==None or self.type_axes!=type_axes:
                self.ifi.NotifyEvent(self, "changedata", map)
                self.type_axes=type_axes[:]
            else:
                self.ifi.NotifyEvent(self, "updatedata", map) 
                self.type_axes=type_axes[:] ##[inamura 160823]

        self.ifi.NotifyEvent(self, "title", (labels[0], labels[1]))
        self.ifi.NotifyEvent(self, "scalelabel", (labels[2], labels[3], labels[4]))

        self.plotNow = (map, labels[0], labels[1])  
 
                
    #########################################
    def OnNotify2DClose(self, *args):
        """
        2Dプロッタクローズイベント受信処理
        @param evt　　イベント
        @retval 無し
        """
        self.d2 = None

    #########################################
    def Request2DClose(self):
        """
        2Dプロッタクローズ要求処理
        @param  無し
        @retval 無し
        """
         # 2Dプロッタが開いているなら
        if self.d2 == None:
             # 2Dプロッタのクローズ要求を出す　
            self.ifi.NotifyEvent(self, "close")


#######################################
#  Splash 
#######################################
class UtsusemiSplashScreen(wx.SplashScreen):
    def __init__(self,splash_path):

        bmp = wx.Image(splash_path).ConvertToBitmap()
        wx.SplashScreen.__init__(self,bmp,
                                 wx.SPLASH_CENTRE_ON_SCREEN|wx.SPLASH_TIMEOUT,
                                 5000, None, -1)
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        self.fc = wx.FutureCall( 2000, self.ShowMain)

    def OnClose(self, evt):
        evt.Skip()
        self.Hide()
        if self.fc.IsRunning():
            self.fc.Stop()
            self.ShowMain()

    def ShowMain(self):
        pass


###############################
if __name__=='__main__':
    app = wx.App(False)
    #splash_path = os.path.join( os.environ["UTSUSEMI_BASE_DIR"],"ana","Reduction","LOGO_utsusemi_s.png" )
    splash_path = os.path.join( os.environ["UTSUSEMI_BASE_DIR"],"ana","Reduction","LOGO_utsusemi_3_c_short.png" )
    if os.path.exists(splash_path):
        splash = UtsusemiSplashScreen(splash_path)
        splash.Show()
    
    frame = D4MatSlicer(None)
    app.MainLoop()
